from dataclasses import dataclass


@dataclass
class TokenResponse:
    access_token: str
    refresh_token: str


@dataclass
class CreditResponse:
    free: int
    reward: int
    contract: int
    paid: int
    total: int


@dataclass
class UserDetailResponse:
    first_name: str
    last_name: str
    company: str


@dataclass
class UserResponse:
    user_id: str
    email: str
    detail_data: UserDetailResponse
    credit_info: CreditResponse



from dataclasses import asdict

import requests
from requests_toolbelt import MultipartEncoderMonitor
from requests_toolbelt.multipart.encoder import MultipartEncoder
from tqdm import tqdm

from netspresso.clients.compressor.v2.schemas.common import RequestPagination, UploadFile
from netspresso.clients.compressor.v2.schemas.compression import (
    RequestAutomaticCompressionParams,
    RequestAvailableLayers,
    RequestCreateCompression,
    RequestCreateRecommendation,
    RequestUpdateCompression,
    ResponseCompressionItem,
    ResponseCompressionItems,
    ResponseRecommendationItem,
    ResponseSelectMethodItem,
)
from netspresso.clients.compressor.v2.schemas.model import (
    RequestCreateModel,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelUrl,
)
from netspresso.clients.config import Config, ServiceModule, ServiceName
from netspresso.clients.utils.common import create_multipart_data, create_progress_func, get_headers, progress_callback
from netspresso.clients.utils.requester import Requester


class CompressorAPIClient:
    def __init__(self):
        self.config = Config(ServiceName.NP, ServiceModule.COMPRESSOR)
        self.host = self.config.HOST
        self.port = self.config.PORT
        self.prefix = self.config.URI_PREFIX
        self.url = f"{self.host}:{self.port}{self.prefix}"

    def is_cloud(self) -> bool:
        # TODO
        return self.config.is_cloud()

    def create_model(
        self, request_data: RequestCreateModel, access_token: str, verify_ssl: bool = True
    ) -> ResponseModelUrl:
        url = f"{self.url}/models"
        response = Requester.post_as_json(url=url, request_body=asdict(request_data), headers=get_headers(access_token))

        return ResponseModelUrl(**response.json())

    def upload_model(
        self, request_data: RequestUploadModel, file: UploadFile, access_token: str, verify_ssl: bool = True
    ) -> bool:
        url = f"{self.url}/models/upload"

        file_info = file.files[0][1]

        multipart_data = create_multipart_data(request_data.url, file_info)
        progress = create_progress_func(multipart_data)

        # Wrap the encoder with MultipartEncoderMonitor
        monitor = MultipartEncoderMonitor(multipart_data, lambda monitor: progress_callback(monitor, progress))

        headers = get_headers(access_token)
        headers["Content-Type"] = monitor.content_type

        response = requests.post(url=url, data=monitor, headers=headers, verify=verify_ssl)

        return response.text

    def validate_model(
        self, ai_model_id: str, request_data: RequestValidateModel, access_token: str, verify_ssl: bool = True
    ) -> ResponseModelItem:
        url = f"{self.url}/models/{ai_model_id}/validate"
        response = Requester.post_as_json(
            url=url,
            request_body=asdict(request_data),
            headers=get_headers(access_token),
            timeout=600,
        )

        return ResponseModelItem(**response.json())

    def read_models(
        self, request_params: RequestPagination, access_token: str, verify_ssl: bool = True
    ) -> ResponseModelItems:
        url = f"{self.url}/models"
        response = Requester.get(
            url=url,
            params=asdict(request_params),
            headers=get_headers(access_token),
        )

        return ResponseModelItems(**response.json())

    def read_model(self, ai_model_id: str, access_token: str, verify_ssl: bool = True) -> ResponseModelItem:
        url = f"{self.url}/models/{ai_model_id}"
        response = Requester.get(
            url=url,
            headers=get_headers(access_token),
        )

        return ResponseModelItem(**response.json())

    def download_model(self, ai_model_id: str, access_token: str, verify_ssl: bool = True) -> ResponseModelUrl:
        url = f"{self.url}/models/{ai_model_id}/download"
        response = Requester.post_as_json(
            url=url,
            headers=get_headers(access_token),
        )

        return ResponseModelUrl(**response.json())

    def create_compression(
        self, request_data: RequestCreateCompression, access_token: str, verify_ssl: bool = True
    ) -> ResponseCompressionItem:
        url = f"{self.url}/compressions"
        response = Requester.post_as_json(
            url=url,
            request_body=asdict(request_data),
            headers=get_headers(access_token),
            timeout=600,
        )

        return ResponseCompressionItem(**response.json())

    def read_compressions(
        self, request_params: RequestPagination, access_token: str, verify_ssl: bool = True
    ) -> ResponseCompressionItems:
        url = f"{self.url}/compressions"
        response = Requester.get(
            url=url,
            params=asdict(request_params),
            headers=get_headers(access_token),
        )

        return ResponseCompressionItems(**response.json())

    def read_compression(
        self, compression_id: str, access_token: str, verify_ssl: bool = True
    ) -> ResponseCompressionItem:
        url = f"{self.url}/compressions/{compression_id}"
        response = Requester.get(
            url=url,
            headers=get_headers(access_token),
        )

        return ResponseCompressionItem(**response.json())

    def create_recommendation(
        self, compression_id: str, request_data: RequestCreateRecommendation, access_token: str, verify_ssl: bool = True
    ) -> ResponseRecommendationItem:
        url = f"{self.url}/compressions/{compression_id}/recommendation"
        response = Requester.post_as_json(
            url=url,
            request_body=asdict(request_data),
            headers=get_headers(access_token),
            timeout=600,
        )

        return ResponseRecommendationItem(**response.json())

    def compress_model(
        self, compression_id: str, request_data: RequestUpdateCompression, access_token: str, verify_ssl: bool = True
    ) -> ResponseCompressionItem:
        url = f"{self.url}/compressions/{compression_id}"
        response = Requester.put(
            url=url,
            request_body=asdict(request_data),
            headers=get_headers(access_token),
            timeout=600,
        )

        return ResponseCompressionItem(**response.json())

    def compress_model_with_automatic(
        self,
        ai_model_id: str,
        request_data: RequestAutomaticCompressionParams,
        access_token: str,
        verify_ssl: bool = True,
    ) -> ResponseCompressionItem:
        url = f"{self.url}/models/{ai_model_id}/auto_compress"
        response = Requester.post_as_json(
            url=url,
            request_body=asdict(request_data),
            headers=get_headers(access_token),
            timeout=600,
        )

        return ResponseCompressionItem(**response.json())

    def get_available_layers(
        self, ai_model_id: str, request_data: RequestAvailableLayers, access_token: str, verify_ssl: bool = True
    ) -> ResponseSelectMethodItem:
        url = f"{self.url}/models/{ai_model_id}/available_layers"
        response = Requester.post_as_json(
            url=url,
            request_body=asdict(request_data),
            headers=get_headers(access_token),
        )

        return ResponseSelectMethodItem(**response.json())

    def upload_dataset(self, compression_id: str, file: UploadFile, access_token: str, verify_ssl: bool = True):
        url = f"{self.url}/compressions/{compression_id}/datasets"
        response = Requester.post_as_form(
            url=url,
            binary=file.files,
            headers=get_headers(access_token),
        )

        return ResponseCompressionItem(**response.json())


compressor_client_v2 = CompressorAPIClient()



import dataclasses
from dataclasses import dataclass, field
from typing import List, Optional, Union

from netspresso.clients.launcher.v2.schemas import InputLayer, ResponseItem, ResponseItems
from netspresso.clients.launcher.v2.schemas.task.common import TaskStatusInfo
from netspresso.enums import QuantizationMode, QuantizationPrecision, SimilarityMetric, TaskStatusForDisplay
from netspresso.metadata.quantizer import QuantizeInfo


@dataclass
class QuantizeOption:
    threshold: Union[float, int]
    metric: SimilarityMetric = field(default=SimilarityMetric.SNR)
    weight_precision: QuantizationPrecision = field(default=QuantizationPrecision.INT8)
    activation_precision: QuantizationPrecision = field(default=QuantizationPrecision.INT8)


@dataclass
class QuantizeTask:
    quantize_task_id: str
    input_model_id: str
    output_model_id: str

    quantization_mode: QuantizationMode = field(default=QuantizationMode.UNIFORM_PRECISION_QUANTIZATION)
    options: Optional[QuantizeOption] = field(default_factory=QuantizeOption)
    task_input_layers: List[InputLayer] = field(default_factory=[])

    status: TaskStatusForDisplay = ""
    error_log: Optional[dict] = None

    def __init__(self, **kwargs):
        names = {f.name for f in dataclasses.fields(self)}
        for k, v in kwargs.items():
            if k in names:
                setattr(self, k, v)

        self.task_input_layers = [InputLayer(**task_input_layers) for task_input_layers in self.task_input_layers]
        self.options = QuantizeOption(**self.options)

    def to(self, model_file_name: str) -> QuantizeInfo:
        quantize_info = QuantizeInfo()
        quantize_info.quantize_task_uuid = self.quantize_task_id
        quantize_info.input_model_uuid = self.input_model_id
        quantize_info.output_model_uuid = self.output_model_id
        quantize_info.model_file_name = model_file_name
        quantize_info.quantization_mode = self.quantization_mode
        quantize_info.metric = self.options.metric
        quantize_info.threshold = self.options.threshold
        quantize_info.weight_precision = self.options.weight_precision
        quantize_info.activation_precision = self.options.activation_precision

        return quantize_info


@dataclass
class ResponseQuantizeTaskItem(ResponseItem):
    data: Optional[QuantizeTask] = field(default_factory=dict)

    def __post_init__(self):
        self.data = QuantizeTask(**self.data)


@dataclass
class ResponseQuantizeOptionItems(ResponseItems):
    data: List[Optional[QuantizeOption]] = field(default_factory=list)

    def __post_init__(self):
        self.data = [QuantizeOption(**item) for item in self.data]


@dataclass
class ResponseQuantizeStatusItem(ResponseItem):
    data: TaskStatusInfo = field(default_factory=TaskStatusInfo)

    def __post_init__(self):
        self.data = TaskStatusInfo(**self.data)


@dataclass
class DownloadModelUrl:
    ai_model_id: str
    presigned_download_url: str


@dataclass
class ResponseQuantizeDownloadModelUrlItem(ResponseItem):
    data: Optional[DownloadModelUrl] = field(default_factory=dict)

    def __post_init__(self):
        self.data = DownloadModelUrl(**self.data)



from dataclasses import dataclass
from typing import Optional

from qai_hub import QuantizeDtype

from netspresso.enums.base import StrEnum


class RangeScheme(StrEnum):
    AUTO = "auto"
    MSE_MINIMIZER = "mse_minimizer"
    MIN_MAX = "min_max"


class QuantizePrecision:
    INT8 = QuantizeDtype.INT8
    INT16 = QuantizeDtype.INT16
    INT4 = QuantizeDtype.INT4


@dataclass
class QuantizeOptions:
    """
    Quantize options for the model.

    Note:
        For details, see `QuantizeOptions in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/api.html#quantize-options>`_.
    """

    range_scheme: Optional[RangeScheme] = RangeScheme.AUTO

    def to_cli_string(self) -> str:
        args = []
        if self.range_scheme is not None:
            args.append(f"--range_scheme {self.range_scheme}")

        return " ".join(args)



import time
from pathlib import Path
from typing import Optional, Union
from urllib import request

from loguru import logger

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas import InputLayer
from netspresso.clients.launcher.v2.schemas.common import DeviceInfo
from netspresso.clients.launcher.v2.schemas.task.convert.response_body import ConvertTask
from netspresso.enums import DataType, DeviceName, Framework, ServiceTask, SoftwareVersion, Status, TaskStatusForDisplay
from netspresso.metadata.converter import ConverterMetadata
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class ConverterV2(NetsPressoBase):
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse):
        """Initialize the Converter."""

        super().__init__(token_handler)
        self.user_info = user_info

    def create_available_options(self, target_framework, target_device, target_software_version):
        def filter_device(device: DeviceInfo, target_software_version: SoftwareVersion):
            filtered_versions = [
                version for version in device.software_versions if version.software_version == target_software_version
            ]

            if filtered_versions:
                device.software_versions = filtered_versions
                return device
            return None

        self.token_handler.validate_token()

        available_options = launcher_client_v2.benchmarker.read_framework_options(
            access_token=self.token_handler.tokens.access_token,
            framework=target_framework,
        )

        if target_framework in [Framework.TENSORRT, Framework.DRPAI]:
            for available_option in available_options.data:
                if available_option.framework == target_framework:
                    available_option.devices = [
                        filter_device(device, target_software_version)
                        for device in available_option.devices
                        if device.device_name == target_device
                    ]
                available_option.devices = [device for device in available_option.devices if device]

        return available_options

    def initialize_metadata(
        self, output_dir, input_model_path, target_framework, target_device, target_software_version
    ):
        def create_metadata_with_status(status, error_message=None):
            metadata = ConverterMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = ConverterMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Conversion task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.input_model_path = Path(input_model_path).resolve().as_posix()
            available_options = self.create_available_options(target_framework, target_device, target_software_version)
            metadata.available_options.extend(option.to() for option in available_options.data)
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def _download_converted_model(self, convert_task: ConvertTask, local_path: str) -> None:
        """Download the converted model with given conversion task or conversion task uuid.

        Args:
            conversion_task (ConvertTask): Launcher Model Object or the uuid of the conversion task.

        Raises:
            e: If an error occurs while getting the conversion task information.
        """

        self.token_handler.validate_token()

        try:
            download_url = launcher_client_v2.converter.download_model_file(
                convert_task_uuid=convert_task.convert_task_id,
                access_token=self.token_handler.tokens.access_token,
            ).data.presigned_download_url

            request.urlretrieve(download_url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download converted model failed. Error: {e}")
            raise e

    def convert_model(
        self,
        input_model_path: str,
        output_dir: str,
        target_framework: Union[str, Framework],
        target_device_name: Union[str, DeviceName],
        target_data_type: Union[str, DataType] = DataType.FP16,
        target_software_version: Optional[Union[str, SoftwareVersion]] = None,
        input_layer: Optional[InputLayer] = None,
        dataset_path: Optional[str] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> ConverterMetadata:
        """Convert a model to the specified framework.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the converted model.
            target_framework (Union[str, Framework]): The target framework name.
            target_device_name (Union[str, DeviceName]): Target device name. Required if target_device is not specified.
            target_data_type (Union[str, DataType]): Data type of the model. Default is DataType.FP16.
            target_software_version (Union[str, SoftwareVersion], optional): Target software version.
                Required if target_device_name is one of the Jetson devices.
            input_layer (InputShape, optional): Target input shape for conversion (e.g., dynamic batch to static batch).
            dataset_path (str, optional): Path to the dataset. Useful for certain conversions.
            wait_until_done (bool): If True, wait for the conversion result before returning the function.
                                If False, request the conversion and return  the function immediately.

        Raises:
            e: If an error occurs during the model conversion.

        Returns:
            ConverterMetadata: Convert metadata.
        """

        netspresso_analytics.send_event(
            event_name="convert_model_using_np",
            event_params={
                "target_framework": target_framework,
                "target_device_name": target_device_name,
                "target_data_type": target_data_type,
                "target_software_version": target_software_version or "",
            },
        )

        FileHandler.check_input_model_path(input_model_path)
        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata = self.initialize_metadata(
            output_dir=output_dir,
            input_model_path=input_model_path,
            target_framework=target_framework,
            target_device=target_device_name,
            target_software_version=target_software_version,
        )

        try:
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            self.validate_token_and_check_credit(service_task=ServiceTask.MODEL_CONVERT)

            # Get presigned_model_upload_url
            presigned_url_response = launcher_client_v2.converter.presigned_model_upload_url(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
            )

            # Upload model_file
            launcher_client_v2.converter.upload_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                presigned_upload_url=presigned_url_response.data.presigned_upload_url,
            )

            # Validate model_file
            validate_model_response = launcher_client_v2.converter.validate_model_file(
                access_token=self.token_handler.tokens.access_token,
                input_model_path=input_model_path,
                ai_model_id=presigned_url_response.data.ai_model_id,
            )

            # Start convert task
            convert_response = launcher_client_v2.converter.start_task(
                access_token=self.token_handler.tokens.access_token,
                input_model_id=presigned_url_response.data.ai_model_id,
                target_device_name=target_device_name,
                target_framework=target_framework,
                data_type=target_data_type,
                input_layer=input_layer if input_layer else validate_model_response.data.detail.input_layers[0],
                software_version=target_software_version,
                dataset_path=dataset_path,
            )

            metadata.model_info = validate_model_response.data.to()
            metadata.convert_task_info = convert_response.data.to(validate_model_response.data.uploaded_file_name)
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

            if wait_until_done:
                while True:
                    self.token_handler.validate_token()
                    convert_response = launcher_client_v2.converter.read_task(
                        access_token=self.token_handler.tokens.access_token,
                        task_id=convert_response.data.convert_task_id,
                    )
                    if convert_response.data.status in [
                        TaskStatusForDisplay.FINISHED,
                        TaskStatusForDisplay.ERROR,
                        TaskStatusForDisplay.TIMEOUT,
                    ]:
                        break

                    time.sleep(sleep_interval)

            if convert_response.data.status == TaskStatusForDisplay.FINISHED:
                default_model_path = FileHandler.get_default_model_path(folder_path=output_dir)
                extension = FileHandler.get_extension(framework=target_framework)
                self._download_converted_model(
                    convert_task=convert_response.data,
                    local_path=str(default_model_path.with_suffix(extension)),
                )
                self.print_remaining_credit(service_task=ServiceTask.MODEL_CONVERT)
                metadata.status = Status.COMPLETED
                metadata.converted_model_path = default_model_path.with_suffix(extension).as_posix()
                logger.info("Conversion task was completed successfully.")
            else:
                metadata = self.handle_error(metadata, ServiceTask.MODEL_CONVERT, convert_response.data.error_log)

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.MODEL_CONVERT, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.MODEL_CONVERT)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def get_conversion_task(self, conversion_task_id: str) -> ConvertTask:
        """Get the conversion task information with given conversion task uuid.

        Args:
            conversion_task_id (str): Convert task UUID of the convert task.

        Raises:
            e: If an error occurs during the model conversion.

        Returns:
            ConversionTask: Model conversion task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.converter.read_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=conversion_task_id,
        )
        return response.data

    def cancel_conversion_task(self, conversion_task_id: str) -> ConvertTask:
        """Cancel the conversion task with given conversion task uuid.

        Args:
            conversion_task_id (str): Convert task UUID of the convert task.

        Raises:
            e: If an error occurs during the task cancel.

        Returns:
            ConversionTask: Model conversion task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.converter.cancel_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=conversion_task_id,
        )
        return response.data



from dataclasses import dataclass, field

from netspresso.clients.auth import response_body
from netspresso.clients.auth.v2.schemas.common import AbstractResponse


@dataclass
class TokenPayload:
    access_token: str
    refresh_token: str


@dataclass
class TokenResponse(AbstractResponse):
    access_token: str
    refresh_token: str

    data: TokenPayload = field(default_factory=TokenPayload)

    def __post_init__(self):
        self.data = TokenPayload(**self.data)

    def to(self) -> response_body.TokenResponse:
        return response_body.TokenResponse(access_token=self.data.access_token, refresh_token=self.data.refresh_token)



from netspresso.trainer.training.environment import EnvironmentConfig
from netspresso.trainer.training.logging import LoggingConfig
from netspresso.trainer.training.training import (
    ClassificationScheduleConfig,
    DetectionScheduleConfig,
    ScheduleConfig,
    SegmentationScheduleConfig,
)

TRAINING_CONFIG_TYPE = {
    "classification": ClassificationScheduleConfig,
    "detection": DetectionScheduleConfig,
    "segmentation": SegmentationScheduleConfig,
}


__all__ = ["ScheduleConfig", "TRAINING_CONFIG_TYPE", "EnvironmentConfig", "LoggingConfig"]



import warnings
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from loguru import logger
from omegaconf import OmegaConf

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.launcher import launcher_client_v2
from netspresso.enums import Framework, Optimizer, Scheduler, ServiceTask, Status, Task
from netspresso.exceptions.trainer import (
    BaseDirectoryNotFoundException,
    DirectoryNotFoundException,
    FailedTrainingException,
    FileNotFoundErrorException,
    NotSetDatasetException,
    NotSetModelException,
    NotSupportedModelException,
    NotSupportedTaskException,
    RetrainingFunctionException,
    TaskOrYamlPathException,
)
from netspresso.metadata.common import InputShape
from netspresso.metadata.trainer import TrainerMetadata
from netspresso.trainer.augmentations import AUGMENTATION_CONFIG_TYPE, AugmentationConfig, Transform
from netspresso.trainer.data import DATA_CONFIG_TYPE, ImageLabelPathConfig, PathConfig
from netspresso.trainer.models import (
    CLASSIFICATION_MODELS,
    DETECTION_MODELS,
    SEGMENTATION_MODELS,
    CheckpointConfig,
    ModelConfig,
)
from netspresso.trainer.trainer_configs import TrainerConfigs
from netspresso.trainer.training import TRAINING_CONFIG_TYPE, EnvironmentConfig, LoggingConfig, ScheduleConfig
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class Trainer(NetsPressoBase):
    def __init__(
        self, token_handler: TokenHandler, task: Optional[Union[str, Task]] = None, yaml_path: Optional[str] = None
    ) -> None:
        """Initialize the Trainer.

        Args:
            task (Union[str, Task]], optional): The type of task (classification, detection, segmentation). Either 'task' or 'yaml_path' must be provided, but not both.
            yaml_path (str, optional): Path to the YAML configuration file. Either 'task' or 'yaml_path' must be provided, but not both.
        """

        self.token_handler = token_handler
        self.deprecated_names = {
            "EfficientFormer": "EfficientFormer-L1",
            "MobileNetV3_Small": "MobileNetV3-S",
            "MobileNetV3_Large": "MobileNetV3-L",
            "ViT-Tiny": "ViT-T",
            "MixNet-Small": "MixNet-S",
            "MixNet-Medium": "MixNet-M",
            "MixNet-Large": "MixNet-L",
            "PIDNet": "PIDNet-S",
        }

        if (task is not None) == (yaml_path is not None):
            raise TaskOrYamlPathException()

        if task is not None:
            self._initialize_from_task(task)
        elif yaml_path is not None:
            self._initialize_from_yaml(yaml_path)

    def _initialize_from_task(self, task: Union[str, Task]) -> None:
        """Initialize the Trainer object based on the provided task.

        Args:
            task (Union[str, Task]): The task for which the Trainer is initialized.
        """

        self.task = self._validate_task(task)
        self.available_models = list(self._get_available_models().keys())
        self.data = None
        self.model = None
        self.training = TRAINING_CONFIG_TYPE[self.task]()
        self.augmentation = AUGMENTATION_CONFIG_TYPE[self.task]()
        self.logging = LoggingConfig()
        self.environment = EnvironmentConfig()

    def _initialize_from_yaml(self, yaml_path: str) -> None:
        """Initialize the Trainer object based on the configuration provided in a YAML file.

        Args:
            yaml_path (str): The path to the YAML file containing the configuration.
        """

        hparams = OmegaConf.load(yaml_path)
        hparams["model"].pop("single_task_model")

        metadata_path = Path(yaml_path).parent / "metadata.json"
        metadata = FileHandler.load_json(metadata_path)
        self.model_name = metadata["model_info"]["model"]

        self.img_size = hparams["augmentation"]["img_size"]
        self.task = hparams["data"]["task"]
        self.available_models = list(self._get_available_models().keys())

        self.data = DATA_CONFIG_TYPE[self.task](**hparams["data"])
        self.model = ModelConfig(**hparams["model"])
        self.training = ScheduleConfig(**hparams["training"])
        self.augmentation = AugmentationConfig(**hparams["augmentation"])
        self.logging = LoggingConfig(**hparams["logging"])
        self.environment = EnvironmentConfig(**hparams["environment"])

    def _validate_task(self, task: Union[str, Task]):
        """Validate the provided task.

        Args:
            task (Union[str, Task]): The task to be validated.

        Raises:
            ValueError: If the provided task is not supported.

        Returns:
            Task: The validated task.
        """

        available_tasks = [task.value for task in Task]
        if task not in available_tasks:
            raise NotSupportedTaskException(available_tasks, task)
        return task

    def _validate_config(self):
        """Validate the configuration setup.

        Raises:
            ValueError: Raised if the dataset is not set. Use `set_dataset_config` or `set_dataset_config_with_yaml` to set the dataset configuration.
            ValueError: Raised if the model is not set. Use `set_model_config` or `set_model_config_with_yaml` to set the model configuration.
        """

        if self.data is None:
            raise NotSetDatasetException()
        if self.model is None:
            raise NotSetModelException()

    def _get_available_models(self) -> Dict[str, Any]:
        """Get available models based on the current task.

        Returns:
            Dict[str, Any]: A dictionary mapping model types to available models.
        """

        available_models = {
            "classification": CLASSIFICATION_MODELS,
            "detection": DETECTION_MODELS,
            "segmentation": SEGMENTATION_MODELS,
        }[self.task]

        # Filter out deprecated names
        filtered_models = {
            name: config for name, config in available_models.items() if name not in self.deprecated_names
        }

        return filtered_models

    def _get_available_models_w_deprecated_names(self) -> Dict[str, Any]:
        """Get available models based on the current task.

        Returns:
            Dict[str, Any]: A dictionary mapping model types to available models.
        """

        available_models = {
            "classification": CLASSIFICATION_MODELS,
            "detection": DETECTION_MODELS,
            "segmentation": SEGMENTATION_MODELS,
        }[self.task]

        return available_models

    def set_dataset_config(
        self,
        name: str,
        root_path: str,
        train_image: str = "images/train",
        train_label: str = "labels/train",
        valid_image: str = "images/valid",
        valid_label: str = "labels/valid",
        test_image: str = "images/valid",
        test_label: str = "labels/valid",
        id_mapping: Optional[Union[List[str], Dict[str, str], str]] = None,
    ):
        """Set the dataset configuration for the Trainer.

        Args:
            name (str): The name of dataset.
            root_path (str): Root directory of dataset.
            train_image (str, optional): The directory for training images. Should be relative path to root directory. Defaults to "images/train".
            train_label (str, optional): The directory for training labels. Should be relative path to root directory. Defaults to "labels/train".
            valid_image (str, optional): The directory for validation images. Should be relative path to root directory. Defaults to "images/val".
            valid_label (str, optional): The directory for validation labels. Should be relative path to root directory. Defaults to "labels/val".
            id_mapping (Union[List[str], Dict[str, str]], optional): ID mapping for the dataset. Defaults to None.
        """

        common_config = {
            "name": name,
            "path": PathConfig(
                root=root_path,
                train=ImageLabelPathConfig(image=train_image, label=train_label),
                valid=ImageLabelPathConfig(image=valid_image, label=valid_label),
                test=ImageLabelPathConfig(image=test_image, label=test_label),
            ),
            "id_mapping": id_mapping,
        }
        self.data = DATA_CONFIG_TYPE[self.task](**common_config)

    def check_paths_exist(self, base_path):
        paths = [
            "images/train",
            "images/valid",
            "id_mapping.json",
        ]

        # Check for the existence of required directories and files
        for relative_path in paths:
            path = Path(base_path) / relative_path
            if not path.exists():
                if path.suffix:  # It's a file
                    raise FileNotFoundErrorException(relative_path)
                else:  # It's a directory
                    raise DirectoryNotFoundException(relative_path)

    def find_paths(self, base_path: str, search_dir, split: str) -> List[str]:
        base_dir = Path(base_path)

        if not base_dir.exists():
            raise BaseDirectoryNotFoundException(base_dir)

        result_paths = []

        dir_path = base_dir / search_dir
        if dir_path.exists() and dir_path.is_dir():
            for item in dir_path.iterdir():
                if (item.is_dir() or item.is_file()) and split in item.name:
                    result_paths.append(item.as_posix())

        return result_paths[0]

    def set_dataset(self, dataset_root_path: str):
        dataset_name = Path(dataset_root_path).name
        root_path = Path(dataset_root_path).resolve().as_posix()

        self.check_paths_exist(root_path)
        images_train = self.find_paths(root_path, "images", "train")
        images_valid = self.find_paths(root_path, "images", "valid")
        labels_train = self.find_paths(root_path, "labels", "train")
        labels_valid = self.find_paths(root_path, "labels", "valid")
        id_mapping = FileHandler.load_json(f"{root_path}/id_mapping.json")
        self.set_dataset_config(
            name=dataset_name,
            root_path=dataset_root_path,
            train_image=images_train,
            train_label=labels_train,
            valid_image=images_valid,
            valid_label=labels_valid,
            id_mapping=id_mapping,
        )

    def set_model_config(
        self,
        model_name: str,
        img_size: int,
        use_pretrained: bool = True,
        load_head: bool = False,
        path: Optional[str] = None,
        fx_model_path: Optional[str] = None,
        optimizer_path: Optional[str] = None,
    ):
        """Set the model configuration for the Trainer.

        Args:
            model_name (str): Name of the model.
            img_size (int): Image size for the model.
            use_pretrained (bool, optional): Whether to use a pre-trained model. Defaults to True.
            load_head (bool, optional): Whether to load the model head. Defaults to False.
            path (str, optional): Path to the model. Defaults to None.
            fx_model_path (str, optional): Path to the FX model. Defaults to None.
            optimizer_path (str, optional): Path to the optimizer. Defaults to None.

        Raises:
            ValueError: If the specified model is not supported for the current task.
        """

        if model_name in self.deprecated_names:
            warnings.filterwarnings("default", category=DeprecationWarning)
            warnings.warn(
                f"The model name '{model_name}' is deprecated and will be removed in a future version. "
                f"Please use '{self.deprecated_names[model_name]}' instead.",
                DeprecationWarning,
                stacklevel=2,
            )
            warnings.filterwarnings("ignore", category=DeprecationWarning)

        self.model_name = model_name
        model = self._get_available_models_w_deprecated_names().get(model_name)
        self.img_size = img_size
        self.logging.sample_input_size = [img_size, img_size]

        if model is None:
            raise NotSupportedModelException()

        self.model = model(
            checkpoint=CheckpointConfig(
                use_pretrained=use_pretrained,
                load_head=load_head,
                path=path,
                fx_model_path=fx_model_path,
                optimizer_path=optimizer_path,
            )
        )

    def set_fx_model(self, fx_model_path: str):
        """Set the FX model path for retraining.

        Args:
            fx_model_path (str): The path to the FX model.

        Raises:
            ValueError: If the model is not set. Please use 'set_model_config' for model setup.
        """

        if not self.model:
            raise RetrainingFunctionException()

        self.model.checkpoint.path = None
        self.model.checkpoint.fx_model_path = fx_model_path

    def set_training_config(
        self,
        optimizer,
        scheduler,
        epochs: int = 3,
        batch_size: int = 8,
    ):
        """Set the training configuration.

        Args:
            optimizer: The configuration of optimizer.
            scheduler: The configuration of learning rate scheduler.
            epochs (int, optional): The total number of epoch for training the model. Defaults to 3.
            batch_size (int, optional): The number of samples in single batch input. Defaults to 8.
        """

        self.training = ScheduleConfig(
            epochs=epochs,
            optimizer=optimizer.asdict(),
            scheduler=scheduler.asdict(),
        )
        self.environment.batch_size = batch_size

    def set_augmentation_config(
        self,
        train_transforms: Optional[List] = None,
        inference_transforms: Optional[List] = None,
    ):
        """Set the augmentation configuration for training.

        Args:
            train_transforms (List, optional): List of transforms for training. Defaults to None.
            inference_transforms (List, optional): List of transforms for inference. Defaults to None.
        """

        self.augmentation = AugmentationConfig(
            train=train_transforms,
            inference=inference_transforms,
        )

    def set_logging_config(
        self,
        project_id: Optional[str] = None,
        output_dir: str = "./outputs",
        tensorboard: bool = True,
        csv: bool = False,
        image: bool = True,
        stdout: bool = True,
        save_optimizer_state: bool = True,
        validation_epoch: int = 10,
        save_checkpoint_epoch: Optional[int] = None,
    ):
        """Set the logging configuration.

        Args:
            project_id (str, optional): Project name to save the experiment. If None, it is set as {task}_{model} (e.g. segmentation_segformer).
            output_dir (str, optional): Root directory for saving the experiment. Defaults to "./outputs".
            tensorboard (bool, optional): Whether to use the tensorboard. Defaults to True.
            csv (bool, optional): Whether to save the result in csv format. Defaults to False.
            image (bool, optional): Whether to save the validation results. It is ignored if the task is classification. Defaults to True.
            stdout (bool, optional): Whether to log the standard output. Defaults to True.
            save_optimizer_state (bool, optional): Whether to save optimizer state with model checkpoint to resume training. Defaults to True.
            validation_epoch (int, optional): Validation frequency in total training process. Defaults to 10.
            save_checkpoint_epoch (int, optional): Checkpoint saving frequency in total training process. Defaults to None.
        """

        self.logging = LoggingConfig(
            project_id=project_id,
            output_dir=output_dir,
            tensorboard=tensorboard,
            csv=csv,
            image=image,
            stdout=stdout,
            save_optimizer_state=save_optimizer_state,
            validation_epoch=validation_epoch,
            save_checkpoint_epoch=save_checkpoint_epoch,
        )

    def set_environment_config(self, seed: int = 1, num_workers: int = 4):
        """Set the environment configuration.

        Args:
            seed (int, optional): Random seed. Defaults to 1.
            num_workers (int, optional): The number of multi-processing workers to be used by the data loader. Defaults to 4.
        """

        self.environment = EnvironmentConfig(seed=seed, num_workers=num_workers)

    def _change_transforms(self, transforms: Transform):
        """Update the 'size' attribute in the given list of transforms with the specified image size.

        Args:
            transforms (List[Transform]): The list of transforms to be updated.

        Returns:
            List[Transform]: The list of transforms with the 'size' attribute updated to the specified image size.
        """

        field_name_to_check = "size"

        if transforms is None:
            return transforms

        for transform in transforms:
            field_type = transform.__annotations__.get(field_name_to_check)

            if field_type == List:
                transform.size = [self.img_size, self.img_size]
            elif isinstance(field_type, int):
                transform.size = self.img_size

        return transforms

    def _apply_img_size(self):
        """Apply the specified image size to the augmentation configurations.

        This method updates the 'img_size' attribute in the augmentation configurations, including
        'train.transforms', 'train.mix_transforms', and 'inference.transforms'.
        """

        self.augmentation.img_size = self.img_size
        self.augmentation.train = self._change_transforms(self.augmentation.train)
        self.augmentation.inference = self._change_transforms(self.augmentation.inference)

    def _get_available_options(self):
        self.token_handler.validate_token()
        options_response = launcher_client_v2.converter.read_framework_options(
            access_token=self.token_handler.tokens.access_token,
            framework=Framework.ONNX,
        )

        available_options = options_response.data

        # TODO: Will be removed when we support DLC in the future
        available_options = [
            available_option for available_option in available_options if available_option.framework != "dlc"
        ]

        return available_options

    def _get_status_by_training_summary(self, status):
        status_mapping = {
            "success": Status.COMPLETED,
            "stop": Status.STOPPED,
            "error": Status.ERROR,
            "": Status.IN_PROGRESS,
        }
        return status_mapping.get(status, Status.IN_PROGRESS)

    def initialize_metadata(self, output_dir):
        def create_metadata_with_status(status, error_message=None):
            metadata = TrainerMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = TrainerMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Training task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.update_output_dir(output_dir.resolve().as_posix())
            metadata.update_model_info(
                task=self.task,
                model=self.model_name,
                dataset=self.data.name,
                input_shapes=[InputShape(batch=1, channel=3, dimension=[self.img_size, self.img_size])],
            )
            metadata.update_training_info(
                epochs=self.training.epochs,
                batch_size=self.environment.batch_size,
                learning_rate=self.training.optimizer["lr"],
                optimizer=Optimizer.to_display_name(self.training.optimizer["name"]),
                scheduler=Scheduler.to_display_name(self.training.scheduler["name"]),
            )
            metadata.update_hparams(hparams=(output_dir / "hparams.yaml").resolve().as_posix())
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def find_best_model_paths(self, destination_folder: Path):
        best_fx_paths_set = set()

        for pattern in ["*best_fx.pt", "*best.pt"]:
            best_fx_paths_set.update(destination_folder.glob(pattern))

        best_fx_paths = list(best_fx_paths_set)
        best_onnx_paths = list(destination_folder.glob("*best.onnx"))

        return best_fx_paths, best_onnx_paths

    def create_runtime_config(self, yaml_path):
        hparams = OmegaConf.load(yaml_path)

        preprocess = hparams.augmentation.inference
        for _preprocess in preprocess:
            if hasattr(_preprocess, "size") and _preprocess.size:
                _preprocess.size = _preprocess.size[0]
            if _preprocess.name == "resize":
                _preprocess.resize_criteria = "long"

        if hparams.model.task == Task.IMAGE_CLASSIFICATION:
            visualize = {"params": {"class_map": hparams.data.id_mapping, "pallete": None}}
        elif hparams.model.task == Task.OBJECT_DETECTION:
            visualize = {
                "params": {"class_map": hparams.data.id_mapping, "normalized": False, "brightness_factor": 1.5}
            }
        elif hparams.model.task == Task.SEMANTIC_SEGMENTATION:
            visualize = {
                "params": {
                    "class_map": hparams.data.id_mapping,
                    "pallete": None,
                    "normalized": False,
                    "brightness_factor": 1.5,
                }
            }

        _config = {
            "task": hparams.model.task,
            "preprocess": preprocess,
            "postprocess": hparams.model.postprocessor,
            "visualize": visualize,
        }
        name = "runtime"
        config = OmegaConf.create({name: _config})
        save_path = Path(yaml_path).parent / f"{name}.yaml"
        OmegaConf.save(config=config, f=save_path)

        return save_path

    def train(self, gpus: str, project_name: str, output_dir: Optional[str] = "./outputs") -> TrainerMetadata:
        """Train the model with the specified configuration.

        Args:
            gpus (str): GPU ids to use, separated by commas.
            project_name (str): Project name to save the experiment.

        Returns:
            Dict: A dictionary containing information about the training.
        """

        from netspresso_trainer import train_with_yaml

        netspresso_analytics.send_event(
            event_name="train_model_using_np",
            event_params={
                "task": self.task,
                "model": self.model_name,
            },
        )

        self._validate_config()
        self._apply_img_size()

        project_name = project_name if project_name else f"{self.task}_{self.model_name}".lower()
        destination_folder = Path(output_dir) / project_name
        destination_folder = FileHandler.create_unique_folder(folder_path=destination_folder)
        metadata = self.initialize_metadata(output_dir=destination_folder)

        try:
            self.logging.output_dir = output_dir
            self.logging.project_id = destination_folder.name
            self.logging_dir = Path(self.logging.output_dir) / self.logging.project_id / "version_0"
            self.environment.gpus = gpus

            configs = TrainerConfigs(
                self.data,
                self.augmentation,
                self.model,
                self.training,
                self.logging,
                self.environment,
            )
            train_with_yaml(
                gpus=gpus,
                data=configs.data,
                augmentation=configs.augmentation,
                model=configs.model,
                training=configs.training,
                logging=configs.logging,
                environment=configs.environment,
            )

            available_options = self._get_available_options()
            metadata.update_available_options(available_options)

        except Exception as e:
            e = FailedTrainingException(error_log=e.args[0])
            metadata = self.handle_error(metadata, ServiceTask.TRAINING, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.TRAINING)
        finally:
            FileHandler.remove_folder(configs.temp_folder)
            logger.info(f"Removed {configs.temp_folder} folder.")

            FileHandler.move_and_cleanup_folders(source_folder=self.logging_dir, destination_folder=destination_folder)
            logger.info(f"Files in {self.logging_dir} were moved to {destination_folder}.")

            runtime_config_path = self.create_runtime_config(yaml_path=destination_folder / "hparams.yaml")
            metadata.runtime = runtime_config_path.as_posix()

            training_summary = FileHandler.load_json(file_path=destination_folder / "training_summary.json")
            metadata.update_training_result(training_summary=training_summary)

            status = self._get_status_by_training_summary(training_summary.get("status"))
            metadata.update_status(status=status)
            if status == Status.ERROR:
                error_stats = training_summary.get("error_stats", "")
                e = FailedTrainingException(error_log=error_stats)
                metadata.update_message(exception_detail=e.args[0])

            best_fx_paths, best_onnx_paths = self.find_best_model_paths(destination_folder)
            if best_fx_paths:
                metadata.update_best_fx_model_path(best_fx_model_path=best_fx_paths[0].resolve().as_posix())
            if best_onnx_paths:
                metadata.update_best_onnx_model_path(best_onnx_model_path=best_onnx_paths[0].resolve().as_posix())
            MetadataHandler.save_metadata(data=metadata, folder_path=destination_folder)

        return metadata



from netspresso.enums.base import StrEnum


class QuantizationPrecision(StrEnum):
    INT8 = "int8"
    FLOAT16 = "float16"
    FLOAT32 = "float32"


class QuantizationMode(StrEnum):
    AUTOMATIC_QUANTIZATION = "automatic_quantization"
    UNIFORM_PRECISION_QUANTIZATION = "uniform_precision_quantization"
    CUSTOM_PRECISION_QUANTIZATION = "custom_precision_quantization"
    ADVANCED_QUANTIZATION = "advanced_quantization"
    RECOMMEND_QUANTIZATION = "recommend_quantization"


class SimilarityMetric(StrEnum):
    SNR = "SNR"


class OnnxOperator(StrEnum):
    Adagrad = "Adagrad"
    Momentum = "Momentum"
    ZipMap = "ZipMap"
    TreeEnsembleClassifier = "TreeEnsembleClassifier"
    SVMRegressor = "SVMRegressor"
    LinearClassifier = "LinearClassifier"
    DictVectorizer = "DictVectorizer"
    CategoryMapper = "CategoryMapper"
    CastMap = "CastMap"
    StringSplit = "StringSplit"
    Adam = "Adam"
    Gelu = "Gelu"
    Scaler = "Scaler"
    AffineGrid = "AffineGrid"
    BitwiseOr = "BitwiseOr"
    BitwiseAnd = "BitwiseAnd"
    Mish = "Mish"
    MelWeightMatrix = "MelWeightMatrix"
    BlackmanWindow = "BlackmanWindow"
    Gradient = "Gradient"
    HammingWindow = "HammingWindow"
    HannWindow = "HannWindow"
    SequenceMap = "SequenceMap"
    LayerNormalization = "LayerNormalization"
    GridSample = "GridSample"
    CastLike = "CastLike"
    OptionalHasElement = "OptionalHasElement"
    Optional = "Optional"
    SoftmaxCrossEntropyLoss = "SoftmaxCrossEntropyLoss"
    GreaterOrEqual = "GreaterOrEqual"
    LessOrEqual = "LessOrEqual"
    Celu = "Celu"
    Imputer = "Imputer"
    SequenceLength = "SequenceLength"
    OptionalGetElement = "OptionalGetElement"
    SequenceAt = "SequenceAt"
    SequenceConstruct = "SequenceConstruct"
    GatherND = "GatherND"
    ScatterND = "ScatterND"
    GatherElements = "GatherElements"
    CumSum = "CumSum"
    RoiAlign = "RoiAlign"
    NonMaxSuppression = "NonMaxSuppression"
    DequantizeLinear = "DequantizeLinear"
    DFT = "DFT"
    QuantizeLinear = "QuantizeLinear"
    QLinearConv = "QLinearConv"
    ConvInteger = "ConvInteger"
    Det = "Det"
    QLinearMatMul = "QLinearMatMul"
    MatMulInteger = "MatMulInteger"
    Normalizer = "Normalizer"
    RandomNormal = "RandomNormal"
    IsInf = "IsInf"
    Or = "Or"
    ConcatFromSequence = "ConcatFromSequence"
    Constant = "Constant"
    FeatureVectorizer = "FeatureVectorizer"
    Mul = "Mul"
    Max = "Max"
    GRU = "GRU"
    GroupNormalization = "GroupNormalization"
    Mod = "Mod"
    Log = "Log"
    ArgMax = "ArgMax"
    ReduceMax = "ReduceMax"
    Split = "Split"
    If = "If"
    ReduceMin = "ReduceMin"
    DeformConv = "DeformConv"
    MaxPool = "MaxPool"
    Sign = "Sign"
    Min = "Min"
    Range = "Range"
    Cast = "Cast"
    PRelu = "PRelu"
    RandomUniform = "RandomUniform"
    Pad = "Pad"
    NonZero = "NonZero"
    Ceil = "Ceil"
    Tan = "Tan"
    Not = "Not"
    Clip = "Clip"
    ReduceL2 = "ReduceL2"
    Neg = "Neg"
    LinearRegressor = "LinearRegressor"
    BitwiseXor = "BitwiseXor"
    Conv = "Conv"
    StringConcat = "StringConcat"
    Abs = "Abs"
    Softplus = "Softplus"
    MaxUnpool = "MaxUnpool"
    TfIdfVectorizer = "TfIdfVectorizer"
    ConvTranspose = "ConvTranspose"
    SequenceEmpty = "SequenceEmpty"
    And = "And"
    Flatten = "Flatten"
    ReduceLogSum = "ReduceLogSum"
    Einsum = "Einsum"
    ReduceLogSumExp = "ReduceLogSumExp"
    Sub = "Sub"
    Floor = "Floor"
    RandomUniformLike = "RandomUniformLike"
    MaxRoiPool = "MaxRoiPool"
    Concat = "Concat"
    Sigmoid = "Sigmoid"
    SequenceInsert = "SequenceInsert"
    Less = "Less"
    Softmax = "Softmax"
    Add = "Add"
    DynamicQuantizeLinear = "DynamicQuantizeLinear"
    InstanceNormalization = "InstanceNormalization"
    Loop = "Loop"
    LpPool = "LpPool"
    ArgMin = "ArgMin"
    Equal = "Equal"
    LabelEncoder = "LabelEncoder"
    SplitToSequence = "SplitToSequence"
    Round = "Round"
    Unique = "Unique"
    BitShift = "BitShift"
    AveragePool = "AveragePool"
    Slice = "Slice"
    Dropout = "Dropout"
    Exp = "Exp"
    EyeLike = "EyeLike"
    LSTM = "LSTM"
    ArrayFeatureExtractor = "ArrayFeatureExtractor"
    MatMul = "MatMul"
    LeakyRelu = "LeakyRelu"
    ReduceMean = "ReduceMean"
    ReverseSequence = "ReverseSequence"
    BatchNormalization = "BatchNormalization"
    LpNormalization = "LpNormalization"
    Gemm = "Gemm"
    OneHot = "OneHot"
    Bernoulli = "Bernoulli"
    NegativeLogLikelihoodLoss = "NegativeLogLikelihoodLoss"
    Greater = "Greater"
    RNN = "RNN"
    GlobalLpPool = "GlobalLpPool"
    Gather = "Gather"
    HardSwish = "HardSwish"
    Mean = "Mean"
    IsNaN = "IsNaN"
    RegexFullMatch = "RegexFullMatch"
    Asin = "Asin"
    OneHotEncoder = "OneHotEncoder"
    DepthToSpace = "DepthToSpace"
    Div = "Div"
    Softsign = "Softsign"
    GlobalMaxPool = "GlobalMaxPool"
    Reciprocal = "Reciprocal"
    MeanVarianceNormalization = "MeanVarianceNormalization"
    ReduceL1 = "ReduceL1"
    Relu = "Relu"
    ScatterElements = "ScatterElements"
    ReduceSum = "ReduceSum"
    Elu = "Elu"
    Reshape = "Reshape"
    ImageDecoder = "ImageDecoder"
    Shape = "Shape"
    Selu = "Selu"
    ConstantOfShape = "ConstantOfShape"
    GlobalAveragePool = "GlobalAveragePool"
    Size = "Size"
    HardSigmoid = "HardSigmoid"
    LogSoftmax = "LogSoftmax"
    SpaceToDepth = "SpaceToDepth"
    TreeEnsembleRegressor = "TreeEnsembleRegressor"
    BitwiseNot = "BitwiseNot"
    ReduceSumSquare = "ReduceSumSquare"
    Sqrt = "Sqrt"
    Col2Im = "Col2Im"
    RandomNormalLike = "RandomNormalLike"
    Multinomial = "Multinomial"
    Identity = "Identity"
    Expand = "Expand"
    Squeeze = "Squeeze"
    STFT = "STFT"
    Sum = "Sum"
    Upsample = "Upsample"
    Tanh = "Tanh"
    Tile = "Tile"
    TopK = "TopK"
    LRN = "LRN"
    Unsqueeze = "Unsqueeze"
    ThresholdedRelu = "ThresholdedRelu"
    SequenceErase = "SequenceErase"
    Xor = "Xor"
    Acos = "Acos"
    Pow = "Pow"
    Atan = "Atan"
    SVMClassifier = "SVMClassifier"
    Cos = "Cos"
    Sin = "Sin"
    Transpose = "Transpose"
    ReduceProd = "ReduceProd"
    Scan = "Scan"
    Compress = "Compress"
    Scatter = "Scatter"
    CenterCropPad = "CenterCropPad"
    Sinh = "Sinh"
    Asinh = "Asinh"
    Binarizer = "Binarizer"
    Trilu = "Trilu"
    Acosh = "Acosh"
    Cosh = "Cosh"
    StringNormalizer = "StringNormalizer"
    Atanh = "Atanh"
    Shrink = "Shrink"
    Hardmax = "Hardmax"
    ERF = "Erf"
    Where = "Where"
    Resize = "Resize"



from dataclasses import asdict

from netspresso.clients.launcher.v2.interfaces import TaskInterface
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    RequestBenchmark,
    ResponseBenchmarkFrameworkOptionItems,
    ResponseBenchmarkOptionItems,
    ResponseBenchmarkStatusItem,
    ResponseBenchmarkTaskItem,
    UploadFile,
)
from netspresso.clients.utils.requester import Requester
from netspresso.enums import LauncherTask


class BenchmarkTaskAPI(TaskInterface):
    def __init__(self, url):
        self.task_type = LauncherTask.BENCHMARK.value
        self.base_url = url
        self.task_base_url = f"{self.base_url}/{self.task_type}/tasks"
        self.option_base_url = f"{self.base_url}/{self.task_type}/options"

    def start(
        self,
        request_body: RequestBenchmark,
        headers: AuthorizationHeader,
        file: UploadFile = None,
    ) -> ResponseBenchmarkTaskItem:
        endpoint = f"{self.task_base_url}"
        response = Requester().post_as_json(url=endpoint, request_body=asdict(request_body), headers=headers.to_dict())
        return ResponseBenchmarkTaskItem(**response.json())

    def cancel(self, headers: AuthorizationHeader, task_id: str) -> ResponseBenchmarkTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}/cancel"
        response = Requester().post_as_json(url=endpoint, request_body={}, headers=headers.to_dict())
        return ResponseBenchmarkTaskItem(**response.json())

    def read(self, headers: AuthorizationHeader, task_id: str) -> ResponseBenchmarkTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseBenchmarkTaskItem(**response.json())

    def delete(self, headers: AuthorizationHeader, task_id: str) -> ResponseBenchmarkTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().delete(url=endpoint, headers=headers.to_dict())
        return ResponseBenchmarkTaskItem(**response.json())

    def status(self, headers: AuthorizationHeader, task_id: str) -> ResponseBenchmarkStatusItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseBenchmarkStatusItem(**response.json())

    def options(self, headers: AuthorizationHeader) -> ResponseBenchmarkOptionItems:
        endpoint = f"{self.option_base_url}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseBenchmarkOptionItems(**response.json())

    def options_by_model_framework(
        self, headers: AuthorizationHeader, model_framework: str
    ) -> ResponseBenchmarkFrameworkOptionItems:
        endpoint = f"{self.option_base_url}/frameworks/{model_framework}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseBenchmarkFrameworkOptionItems(**response.json())

    def option_by_target_device(self, headers: AuthorizationHeader, target_device: str) -> ResponseBenchmarkOptionItems:
        endpoint = f"{self.option_base_url}/details/{target_device}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseBenchmarkOptionItems(**response.json())



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, ModelConfig


@dataclass
class ViTTinyArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "vit",
            "params": {
                "patch_size": 16,
                "attention_channels": 192,
                "num_blocks": 12,
                "num_attention_heads": 3,
                "attention_dropout_prob": 0.0,
                "ffn_intermediate_channels": 768,
                "ffn_dropout_prob": 0.1,
                "use_cls_token": True,
                "vocab_size": 1000,
            },
            "stage_params": None,
        }
    )


@dataclass
class ClassificationViTTinyModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "vit_tiny"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ViTTinyArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )



from dataclasses import dataclass
from typing import List

from netspresso.enums import QuantizationPrecision


@dataclass
class PrecisionByLayer:
    name: str
    precision: QuantizationPrecision


@dataclass
class PrecisionByOperator:
    type: str
    precision: QuantizationPrecision


@dataclass
class RecommendationPrecisions:
    layers: List[PrecisionByLayer]
    operators: List[PrecisionByOperator]



from dataclasses import asdict

import requests
from requests_toolbelt import MultipartEncoderMonitor

from netspresso.clients.launcher.v2.interfaces import ModelInterface
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    RequestModelUploadUrl,
    RequestPagination,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelOptions,
    ResponseModelStatus,
    ResponseModelUploadUrl,
    UploadFile,
)
from netspresso.clients.utils.common import create_multipart_data, create_progress_func, progress_callback
from netspresso.clients.utils.requester import Requester
from netspresso.enums import LauncherTask


class ModelAPI(ModelInterface):
    def __init__(self, url: str, task_type: LauncherTask):
        self.task_type = task_type
        self.base_url = url
        self.model_base_url = f"{self.base_url}/models"

    def get_upload_url(
        self, request_params: RequestModelUploadUrl, headers: AuthorizationHeader
    ) -> ResponseModelUploadUrl:
        endpoint = f"{self.model_base_url}/upload_url"
        response = Requester().get(url=endpoint, headers=headers.to_dict(), params=asdict(request_params))
        return ResponseModelUploadUrl(**response.json())

    def get_download_url(self, headers: AuthorizationHeader, ai_model_id: str) -> str:
        endpoint = f"{self.model_base_url}/{ai_model_id}/download_url"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return response.text

    def upload(
        self,
        request_body: RequestUploadModel,
        file: UploadFile,
        headers: AuthorizationHeader,
    ) -> str:
        url = f"{self.model_base_url}/upload"

        file_info = file.files[0][1]

        multipart_data = create_multipart_data(request_body.url, file_info)
        progress = create_progress_func(multipart_data)

        # Wrap the encoder with MultipartEncoderMonitor
        monitor = MultipartEncoderMonitor(multipart_data, lambda monitor: progress_callback(monitor, progress))

        headers = headers.to_dict()
        headers["Content-Type"] = monitor.content_type

        response = requests.post(url=url, data=monitor, headers=headers)

        return response.text

    def validate(self, request_body: RequestValidateModel, headers: AuthorizationHeader) -> ResponseModelItem:
        endpoint = f"{self.model_base_url}/validate"
        response = Requester().post_as_json(url=endpoint, request_body=asdict(request_body), headers=headers.to_dict())
        return ResponseModelItem(**response.json())

    def read(
        self,
        request_params: RequestPagination,
        headers: AuthorizationHeader,
        ai_model_id: str,
    ) -> ResponseModelItem:
        endpoint = f"{self.model_base_url}/{ai_model_id}"
        response = Requester().get(url=endpoint, params=asdict(request_params), headers=headers.to_dict())
        return ResponseModelItem(**response.json())

    def read_all(self, request_params: RequestPagination, headers: AuthorizationHeader) -> ResponseModelItems:
        endpoint = f"{self.model_base_url}"
        response = Requester().get(url=endpoint, headers=headers.to_dict(), params=asdict(request_params))
        return ResponseModelItems(**response.json())

    def delete(self, headers: AuthorizationHeader, ai_model_id: str) -> ResponseModelItem:
        endpoint = f"{self.model_base_url}/{ai_model_id}"
        response = Requester().delete(url=endpoint, headers=headers.to_dict())
        return ResponseModelItem(**response.json())

    def status(self, headers: AuthorizationHeader, ai_model_id: str) -> ResponseModelStatus:
        endpoint = f"{self.model_base_url}/{ai_model_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseModelStatus(**response.json())

    def options(self, headers: AuthorizationHeader, ai_model_id: str) -> ResponseModelOptions:
        endpoint = f"{self.model_base_url}/{ai_model_id}/options"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseModelOptions(**response.json())



from pathlib import Path
from typing import List, Union

import torch
import torch.nn as nn
from loguru import logger
from torch import Tensor


def _export_onnx(
    model: nn.Module,
    save_path: Union[str, Path],
    sample_input: Tensor,
    opset_version=13,
    input_names="images",
    output_names="output",
):
    torch.onnx.export(
        model,  # model being run
        sample_input,  # model input (or a tuple for multiple inputs)
        save_path,  # where to save the model (can be a file or file-like object)
        export_params=True,  # store the trained parameter weights inside the model file
        opset_version=opset_version,  # the ONNX version to export the model to
        do_constant_folding=True,  # whether to execute constant folding for optimization
        input_names=[input_names],  # the model's input names
        output_names=[output_names],  # the model's output names
        dynamic_axes={input_names: {0: "batch_size"}, output_names: {0: "batch_size"}},  # variable length axes
    )
    logger.info(f"ONNX model converting and saved at {save_path}")


def export_onnx(file_path: str, input_shapes: List[int]):
    file_path = Path(file_path)
    model = torch.load(file_path.with_suffix(".pt"))

    input_shape = input_shapes[0]
    sample_input = torch.randn((1, input_shape.channel, *input_shape.dimension))
    save_dtype = next(model.parameters()).dtype

    return _export_onnx(model, file_path.with_suffix(".onnx"), sample_input=sample_input.type(save_dtype))



from loguru import logger

from netspresso.clients.config import Config, ServiceModule, ServiceName
from netspresso.clients.tao.auth import AuthAPI
from netspresso.clients.tao.dataset import DatasetAPI
from netspresso.clients.tao.experiment import ExperimentAPI
from netspresso.clients.utils.common import create_tao_headers


class TAOAPIClient:
    def __init__(self):
        self.config = Config(ServiceName.TAO, ServiceModule.TRAINER)
        self.host = self.config.HOST
        self.prefix = self.config.URI_PREFIX
        self.url = f"{self.host}{self.prefix}"
        self.auth = self._create_auth_api()
        self.dataset = self._create_dataset_api()
        self.experiment = self._create_experiment_api()

    def _create_auth_api(self):
        return AuthAPI(self.url)

    def _create_dataset_api(self):
        return DatasetAPI(self.url)

    def _create_experiment_api(self):
        return ExperimentAPI(self.url)


class TAOTokenHandler:
    def __init__(self, ngc_api_key: str) -> None:
        try:
            data = {"ngc_api_key": ngc_api_key}
            credentials = tao_client.auth.login(data)
            self.user_id = credentials["user_id"]
            self.token = credentials["token"]
            self.headers = create_tao_headers(self.token)
            logger.info("Login was successfully to TAO.")

        except Exception as e:
            logger.error(f"Login failed. Error: {e}")
            raise e


tao_client = TAOAPIClient()



import os
from typing import List, Optional

from loguru import logger

from netspresso.clients.launcher.v2.implements import ModelAPI, QuantizeTaskAPI
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    InputLayer,
    RequestModelUploadUrl,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelItem,
    ResponseModelOptions,
    ResponseModelUploadUrl,
    ResponseQuantizeDownloadModelUrlItem,
    ResponseQuantizeOptionItems,
    ResponseQuantizeStatusItem,
    ResponseQuantizeTaskItem,
    UploadDataset,
    UploadFile,
)
from netspresso.clients.launcher.v2.schemas.task.quantize.request_body import QuantizationOptions, RequestQuantizeTask
from netspresso.clients.utils.common import read_file_bytes
from netspresso.enums import LauncherTask, QuantizationMode


class Quantizer:
    def __init__(self, url):
        self.task_type = LauncherTask.QUANTIZE.value
        self.quantize_task = QuantizeTaskAPI(url=url)
        self.quantize_model = ModelAPI(url=url, task_type=self.task_type)

    def presigned_model_upload_url(self, access_token: str, input_model_path: str) -> ResponseModelUploadUrl:
        object_name = os.path.basename(input_model_path)

        get_upload_url_request_body = RequestModelUploadUrl(object_name=object_name, task=self.task_type)
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Quantize model upload: path - {input_model_path}")

        upload_url_response_body = self.quantize_model.get_upload_url(
            request_params=get_upload_url_request_body, headers=token_header
        )
        logger.info(f"Request Quantize upload_url result: {upload_url_response_body}")
        return upload_url_response_body

    def upload_model_file(self, access_token: str, input_model_path: str, presigned_upload_url: str) -> str:
        object_name = os.path.basename(input_model_path)
        file_content = read_file_bytes(file_path=input_model_path)

        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(
            f"Request Quantize model validation:"
            f" path | {input_model_path} "
            f" presigned_upload_url | {presigned_upload_url}"
        )

        get_model_upload_request_body = RequestUploadModel(url=presigned_upload_url)
        model_file_object = UploadFile(file_name=object_name, file_content=file_content)

        upload_result = self.quantize_model.upload(
            request_body=get_model_upload_request_body,
            file=model_file_object,
            headers=token_header,
        )
        logger.info(f"Request Quantize upload_model_file result: {upload_result}")
        return upload_result

    def validate_model_file(self, access_token: str, input_model_path: str, ai_model_id: str) -> ResponseModelItem:
        object_name = os.path.basename(input_model_path)
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Quantize model validation:" f" path - {input_model_path}" f" ai_model_id - {ai_model_id}")

        get_validate_model_request_body = RequestValidateModel(
            ai_model_id=ai_model_id,
            display_name=object_name,
        )

        validated_model = self.quantize_model.validate(
            request_body=get_validate_model_request_body, headers=token_header
        )
        logger.info(f"Request Quantize validate_model result: {validated_model}")
        return validated_model

    def download_model_file(self, access_token, quantize_task_uuid) -> ResponseQuantizeDownloadModelUrlItem:
        token_header = AuthorizationHeader(access_token=access_token)
        download_url = self.quantize_task.get_download_url(headers=token_header, quantize_task_uuid=quantize_task_uuid)
        logger.info(f"Request quantizeed model download_url: {download_url}")
        return download_url

    def read_model_task_options(self, access_token, ai_model_id) -> ResponseModelOptions:
        token_header = AuthorizationHeader(access_token=access_token)
        model_task_options = self.quantize_model.options(headers=token_header, ai_model_id=ai_model_id)
        logger.info(f"Request model task_options: {model_task_options}")
        return model_task_options

    def create_upload_dataset(self, dataset_path: str) -> Optional[UploadDataset]:
        if dataset_path:
            dataset_filename = os.path.basename(dataset_path)
            return UploadDataset(
                file_name=dataset_filename,
                file_content=read_file_bytes(dataset_path),
            )
        return None

    def start_task(
        self,
        access_token: str,
        input_model_id: str,
        quantization_mode: QuantizationMode,
        quantization_options: QuantizationOptions,
        input_layers: List[InputLayer] = None,
        dataset_path: str = None,
    ):
        request_body = RequestQuantizeTask(
            input_model_id=input_model_id,
            quantization_mode=quantization_mode,
            quantization_options=quantization_options,
            input_layers=input_layers,
        )
        logger.info(f"Request Quantize body: {request_body}")
        token_header = AuthorizationHeader(access_token=access_token)

        file_object = self.create_upload_dataset(dataset_path)

        if quantization_mode == QuantizationMode.UNIFORM_PRECISION_QUANTIZATION:
            response = self.quantize_task.start_plain_quantization(request_body, token_header, file_object)
        elif quantization_mode == QuantizationMode.RECOMMEND_QUANTIZATION:
            response = self.quantize_task.start_recommendation_precision(request_body, token_header, file_object)
        elif quantization_mode == QuantizationMode.CUSTOM_PRECISION_QUANTIZATION:
            response = self.quantize_task.start_custom_quantization(request_body, token_header, file_object)
        elif quantization_mode == QuantizationMode.AUTOMATIC_QUANTIZATION:
            response = self.quantize_task.start_auto_quantization(request_body, token_header, file_object)

        logger.info(f"Request Quantize result: {response}")
        return response

    def cancel_task(self, access_token: str, task_id: str) -> ResponseQuantizeTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Quantize Cancel task_id: {task_id}")

        quantize_task_response = self.quantize_task.cancel(headers=token_header, task_id=task_id)
        logger.info(f"Request Quantize Cancel result: {quantize_task_response}")
        return quantize_task_response

    def read_task(self, access_token: str, task_id: str) -> ResponseQuantizeTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Quantize Read task_id: {task_id}")

        quantize_task_response = self.quantize_task.read(headers=token_header, task_id=task_id)
        logger.info(f"Request Quantize Task Info: {quantize_task_response}")
        return quantize_task_response

    def delete_task(self, access_token: str, task_id: str) -> ResponseQuantizeTaskItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Quantize delete task_id: {task_id}")

        quantize_task_response = self.quantize_task.delete(headers=token_header, task_id=task_id)
        logger.info(f"Request Quantize Delete Info: {quantize_task_response}")
        return quantize_task_response

    def read_status(self, access_token: str, task_id: str) -> ResponseQuantizeStatusItem:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info(f"Request Quantize read_status task_id: {task_id}")

        quantize_task_response = self.quantize_task.status(headers=token_header, task_id=task_id)
        logger.info(f"Request Quantize Task Status: {quantize_task_response}")
        return quantize_task_response

    def read_options(self, access_token: str) -> ResponseQuantizeOptionItems:
        token_header = AuthorizationHeader(access_token=access_token)
        logger.info("Request Quantize options")

        quantize_task_option_response = self.quantize_task.options(headers=token_header)
        logger.info(f"Response Quantize Task Options: {quantize_task_option_response}")
        return quantize_task_option_response



from typing import Literal

from netspresso.enums.base import StrEnum


class CompressionMethod(StrEnum):
    PR_L2 = "PR_L2"
    PR_GM = "PR_GM"
    PR_NN = "PR_NN"
    PR_SNP = "PR_SNP"
    PR_ID = "PR_ID"
    FD_TK = "FD_TK"
    FD_CP = "FD_CP"
    FD_SVD = "FD_SVD"

    @classmethod
    def create_literal(cls):
        return Literal["PR_L2", "PR_GM", "PR_NN", "PR_SNP", "PR_ID", "FD_TK", "FD_CP", "FD_SVD"]


class RecommendationMethod(StrEnum):
    SLAMP = "slamp"
    VBMF = "vbmf"

    @classmethod
    def create_literal(cls):
        return Literal["slamp", "vbmf"]


class Policy(StrEnum):
    SUM = "sum"
    AVERAGE = "average"

    @classmethod
    def create_literal(cls):
        return Literal["sum", "average"]


class GroupPolicy(StrEnum):
    SUM = "sum"
    AVERAGE = "average"
    COUNT = "count"
    NONE = "none"

    @classmethod
    def create_literal(cls):
        return Literal["sum", "average", "count", "none"]


class LayerNorm(StrEnum):
    NONE = "none"
    STANDARD_SCORE = "standard_score"
    TSS_NORM = "tss_norm"
    LINEAR_SCALING = "linear_scaling"
    SOFTMAX_NORM = "softmax_norm"

    @classmethod
    def create_literal(cls):
        return Literal["none", "standard_score", "tss_norm", "linear_scaling", "softmax_norm"]


class StepOp(StrEnum):
    ROUND_UP = "round_up"
    ROUND_DOWN = "round_down"
    ROUND = "round"
    NONE = "none"

    @classmethod
    def create_literal(cls):
        return Literal["round_up", "round_down", "round", "none"]


compression_literal = CompressionMethod.create_literal()
recommendation_literal = RecommendationMethod.create_literal()
policy_literal = Policy.create_literal()
grouppolicy_literal = GroupPolicy.create_literal()
layernorm_literal = LayerNorm.create_literal()
stepop_literal = StepOp.create_literal()



import cv2


class ClassificationVisualizer:
    def __init__(self, class_map, pallete=None) -> None:
        self.n = len(class_map)
        self.class_map = class_map

    def draw(self, image, pred, model_input_shape=None, text_scale=0.7, text_thickness=2):
        visualize_image = image.copy()

        class_name = self.class_map[int(pred[0])]  # Class is determined with top1 score

        color = (0, 0, 255)
        x1, y1 = 0, 0
        text_size, _ = cv2.getTextSize(str(class_name), cv2.FONT_HERSHEY_SIMPLEX, text_scale, text_thickness)
        text_w, text_h = text_size
        visualize_image = cv2.rectangle(
            visualize_image, (x1, y1), (x1 + text_w, y1 + text_h + 5), color=color, thickness=-1
        )
        visualize_image = cv2.putText(
            visualize_image,
            str(class_name),
            (x1, y1 + text_h),
            cv2.FONT_HERSHEY_SIMPLEX,
            text_scale,
            (255, 255, 255),
            text_thickness,
        )

        return visualize_image



import sys
from enum import Enum

# Check if running on Python 3.11 or later
if sys.version_info >= (3, 11):
    from enum import StrEnum
else:
    # Define a fallback StrEnum for Python 3.10
    class StrEnum(str, Enum):
        pass



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class MobileNetV3SmallArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mobilenetv3",
            "params": None,
            "stage_params": [
                {
                    "in_channels": [16],
                    "kernel_sizes": [3],
                    "expanded_channels": [16],
                    "out_channels": [16],
                    "use_se": [True],
                    "act_type": ["relu"],
                    "stride": [2],
                },
                {
                    "in_channels": [16, 24],
                    "kernel_sizes": [3, 3],
                    "expanded_channels": [72, 88],
                    "out_channels": [24, 24],
                    "use_se": [False, False],
                    "act_type": ["relu", "relu"],
                    "stride": [2, 1],
                },
                {
                    "in_channels": [24, 40, 40, 40, 48],
                    "kernel_sizes": [5, 5, 5, 5, 5],
                    "expanded_channels": [96, 240, 240, 120, 144],
                    "out_channels": [40, 40, 40, 48, 48],
                    "use_se": [True, True, True, True, True],
                    "act_type": ["hard_swish", "hard_swish", "hard_swish", "hard_swish", "hard_swish"],
                    "stride": [2, 1, 1, 1, 1],
                },
                {
                    "in_channels": [48, 96, 96],
                    "kernel_sizes": [5, 5, 5],
                    "expanded_channels": [288, 576, 576],
                    "out_channels": [96, 96, 96],
                    "use_se": [True, True, True],
                    "act_type": ["hard_swish", "hard_swish", "hard_swish"],
                    "stride": [2, 1, 1],
                },
            ],
        }
    )


@dataclass
class MobileNetV3LargeArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "mobilenetv3",
            "params": None,
            "stage_params": [
                {
                    "in_channels": [16, 16, 24],
                    "kernel_sizes": [3, 3, 3],
                    "expanded_channels": [16, 64, 72],
                    "out_channels": [16, 24, 24],
                    "use_se": [False, False, False],
                    "act_type": ["relu", "relu", "relu"],
                    "stride": [1, 2, 1],
                },
                {
                    "in_channels": [24, 40, 40],
                    "kernel_sizes": [5, 5, 5],
                    "expanded_channels": [72, 120, 120],
                    "out_channels": [40, 40, 40],
                    "use_se": [True, True, True],
                    "act_type": ["relu", "relu", "relu"],
                    "stride": [2, 1, 1],
                },
                {
                    "in_channels": [40, 80, 80, 80, 80, 112],
                    "kernel_sizes": [3, 3, 3, 3, 3, 3],
                    "expanded_channels": [240, 200, 184, 184, 480, 672],
                    "out_channels": [80, 80, 80, 80, 112, 112],
                    "use_se": [False, False, False, False, True, True],
                    "act_type": ["hard_swish", "hard_swish", "hard_swish", "hard_swish", "hard_swish", "hard_swish"],
                    "stride": [2, 1, 1, 1, 1, 1],
                },
                {
                    "in_channels": [112, 160, 160],
                    "kernel_sizes": [5, 5, 5],
                    "expanded_channels": [672, 960, 960],
                    "out_channels": [160, 160, 160],
                    "use_se": [True, True, True],
                    "act_type": ["hard_swish", "hard_swish", "hard_swish"],
                    "stride": [2, 1, 1],
                },
            ],
        }
    )


@dataclass
class ClassificationMobileNetV3LargeModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "mobilenet_v3_large"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MobileNetV3LargeArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 2,
                    "intermediate_channels": 1200,
                    "act_type": "hard_swish",
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class ClassificationMobileNetV3SmallModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "mobilenet_v3_small"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MobileNetV3SmallArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class SegmentationMobileNetV3SmallModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "mobilenet_v3_small"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MobileNetV3SmallArchitectureConfig(
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )


@dataclass
class DetectionMobileNetV3SmallModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "mobilenet_v3_small"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MobileNetV3SmallArchitectureConfig(
            neck={
                "name": "fpn",
                "params": {
                    "num_outs": 4,
                    "start_level": 0,
                    "end_level": -1,
                    "add_extra_convs": False,
                    "relu_before_extra_convs": False,
                },
            },
            head={
                "name": "anchor_decoupled_head",
                "params": {
                    # Anchor parameters
                    "anchor_sizes": [
                        [
                            32,
                        ],
                        [
                            64,
                        ],
                        [
                            128,
                        ],
                        [
                            256,
                        ],
                    ],
                    "aspect_ratios": [0.5, 1.0, 2.0],
                    "num_layers": 1,
                    "norm_type": "batch_norm",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "retinanet_loss", "weight": None},
        ]
    )



from netspresso.trainer.data.data import (
    DatasetConfig,
    ImageLabelPathConfig,
    LocalClassificationDatasetConfig,
    LocalDetectionDatasetConfig,
    LocalSegmentationDatasetConfig,
    PathConfig,
)

DATA_CONFIG_TYPE = {
    "classification": LocalClassificationDatasetConfig,
    "detection": LocalDetectionDatasetConfig,
    "segmentation": LocalSegmentationDatasetConfig,
}

__all__ = ["ImageLabelPathConfig", "PathConfig", "DATA_CONFIG_TYPE", "DatasetConfig"]



from dataclasses import dataclass, field
from typing import Dict, Union

from netspresso.enums import QuantizationMode, QuantizationPrecision, SimilarityMetric, TaskType
from netspresso.metadata.common import BaseMetadata, ModelInfo


@dataclass
class QuantizeInfo:
    quantize_task_uuid: str = ""
    model_file_name: str = ""
    quantization_mode: QuantizationMode = QuantizationMode.UNIFORM_PRECISION_QUANTIZATION
    metric: SimilarityMetric = SimilarityMetric.SNR
    threshold: Union[float, int] = 0
    weight_precision: QuantizationPrecision = QuantizationPrecision.INT8
    activation_precision: QuantizationPrecision = QuantizationPrecision.INT8
    input_model_uuid: str = ""
    output_model_uuid: str = ""


@dataclass
class QuantizerMetadata(BaseMetadata):
    task_type: TaskType = TaskType.QUANTIZE
    input_model_path: str = ""
    quantized_model_path: str = ""
    recommendation_result_path: str = ""
    model_info: ModelInfo = field(default_factory=ModelInfo)
    quantize_info: QuantizeInfo = field(default_factory=QuantizeInfo)
    compare_result: Dict = field(default_factory=dict)


@dataclass
class NPQAIQuantizeInfo:
    quantize_task_uuid: str = ""
    model_file_name: str = ""
    input_model_uuid: str = ""
    output_model_uuid: str = ""


@dataclass
class NPQAIQuantizerMetadata(BaseMetadata):
    task_type: TaskType = TaskType.QUANTIZE
    input_model_path: str = ""
    quantized_model_path: str = ""
    model_info: ModelInfo = field(default_factory=ModelInfo)
    quantize_info: NPQAIQuantizeInfo = field(default_factory=NPQAIQuantizeInfo)



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from netspresso.clients.auth import response_body
from netspresso.clients.auth.response_body import UserDetailResponse
from netspresso.clients.auth.v2.schemas.common import AbstractResponse, MembershipType
from netspresso.clients.auth.v2.schemas.credit import SummarizedCreditResponse
from netspresso.clients.auth.v2.schemas.user_agreement import UserAgreementBase


@dataclass
class UserBase:
    username: str
    email: str
    first_name: Optional[str]
    last_name: Optional[str]
    company: Optional[str]
    region: str


@dataclass
class UserPayload(UserBase):
    user_id: str
    is_active: bool
    is_deleted: bool
    is_admin: bool
    is_reset: bool
    last_login_time: datetime
    membership_type: MembershipType

    user_agreement: UserAgreementBase = field(default_factory=UserAgreementBase)

    def __post_init__(self):
        self.user_agreement = UserAgreementBase(**self.user_agreement)


@dataclass
class UserResponse(AbstractResponse):
    data: UserPayload = field(default_factory=UserPayload)

    def __post_init__(self):
        self.data = UserPayload(**self.data)

    def to(self, summarized_credit_response: SummarizedCreditResponse) -> response_body.UserResponse:
        return response_body.UserResponse(
            **{
                "user_id": self.data.user_id,
                "email": self.data.email,
                "detail_data": UserDetailResponse(
                    **{
                        "first_name": self.data.first_name,
                        "last_name": self.data.last_name,
                        "company": self.data.company,
                    }
                ),
                "credit_info": summarized_credit_response.to(),
            }
        )



from .benchmark import (
    RequestBenchmark,
    ResponseBenchmarkFrameworkOptionItems,
    ResponseBenchmarkOptionItems,
    ResponseBenchmarkStatusItem,
    ResponseBenchmarkTaskItem,
)
from .common import TaskStatusInfo
from .convert import (
    RequestConvert,
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    ResponseConvertTaskItem,
)
from .quantize import (
    RequestQuantizeTask,
    ResponseQuantizeDownloadModelUrlItem,
    ResponseQuantizeOptionItems,
    ResponseQuantizeStatusItem,
    ResponseQuantizeTaskItem,
)

__all__ = [
    TaskStatusInfo,
    RequestConvert,
    ResponseConvertTaskItem,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    RequestBenchmark,
    ResponseBenchmarkTaskItem,
    ResponseBenchmarkOptionItems,
    ResponseBenchmarkStatusItem,
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems,
    ResponseBenchmarkFrameworkOptionItems,
    RequestQuantizeTask,
    ResponseQuantizeDownloadModelUrlItem,
    ResponseQuantizeOptionItems,
    ResponseQuantizeStatusItem,
    ResponseQuantizeTaskItem,
]



import subprocess
import sys
from pathlib import Path
from typing import Optional, Union

import pkg_resources
import requests
from loguru import logger
from packaging import version

from netspresso.benchmarker import BenchmarkerV2
from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.tao import TAOTokenHandler
from netspresso.compressor import CompressorV2
from netspresso.converter import ConverterV2
from netspresso.enums import Task
from netspresso.exceptions.common import FailedFetchPackageException
from netspresso.inferencer.inferencer import CustomInferencer, NPInferencer
from netspresso.np_qai.benchmarker import NPQAIBenchmarker
from netspresso.np_qai.converter import NPQAIConverter
from netspresso.np_qai.quantizer import NPQAIQuantizer
from netspresso.quantizer import Quantizer
from netspresso.tao import TAOTrainer
from netspresso.trainer import Trainer
from netspresso.utils.file import FileHandler


class NetsPresso:
    def __init__(self, email: str, password: str, verify_ssl: bool = True, dev_mode: bool = False) -> None:
        """Initialize NetsPresso instance and perform user authentication.

        Args:
            email (str): User's email for authentication.
            password (str): User's password for authentication.
            verify_ssl (bool): Flag to indicate whether SSL certificates should be verified. Defaults to True.
            dev_mode (bool): If True, skip version check for development. Defaults to False.

        Raises:
            SystemExit: If the installed version is not the latest version and dev_mode is False.
        """
        self.dev_mode = dev_mode
        if not self._check_version():
            sys.exit(1)
        self.token_handler = TokenHandler(email=email, password=password, verify_ssl=verify_ssl)
        self.user_info = self.get_user()

    def _check_version(self) -> bool:
        """Check if the installed version of PyNetsPresso is the latest available version.

        Returns:
            bool: True if version check passes or dev_mode is True, False if newer version is available
        """
        if self.dev_mode:
            logger.info("Development mode enabled, skipping version check")
            return True

        try:
            # Get installed version
            current_version = pkg_resources.get_distribution('netspresso').version

            # Get latest version from PyPI
            latest_version = self.get_latest_version('netspresso')

            # Compare versions
            if version.parse(current_version) < version.parse(latest_version):
                logger.warning(
                    f"Your current version is {current_version}. The latest version {latest_version} is released."
                    f"Please upgrade via 'pip install --upgrade netspresso'"
                )
                return False
            return True
        except Exception as e:
            logger.error(f"Failed to check PyNetsPresso version: {str(e)}")
            return False

    def get_latest_version(self, package_name):
        """Get the latest version of a package from PyPI.

        Args:
            package_name (str): Name of the package to check.

        Returns:
            str: Latest version number.

        Raises:
            FailedFetchPackageException: If unable to fetch package information.
        """
        url = f"https://pypi.org/pypi/{package_name}/json"
        response = requests.get(url)
        if response.status_code == 200:
            data = response.json()
            return data["info"]["version"]
        else:
            raise FailedFetchPackageException(package_name=package_name, error_log=response.text)

    def get_user(self) -> UserResponse:
        """Get user information using the access token.

        Returns:
            UserInfo: User information.
        """
        user_info = auth_client.get_user_info(self.token_handler.tokens.access_token, self.token_handler.verify_ssl)
        return user_info

    def create_project(self, project_name: str, project_path: str = "./"):
        # Create the main project folder
        project_folder_path = Path(project_path) / project_name

        # Check if the project folder already exists
        if project_folder_path.exists():
            logger.info(f"Project '{project_name}' already exists at {project_folder_path.resolve()}.")
        else:
            project_folder_path.mkdir(parents=True, exist_ok=True)

            # Subfolder names
            subfolders = ["Trainer models", "Compressed models", "Pretrained models"]

            # Create subfolders
            for folder in subfolders:
                (project_folder_path / folder).mkdir(parents=True, exist_ok=True)

            # Create a metadata.json file
            metadata_file_path = project_folder_path / "metadata.json"
            metadata = {"is_project_folder": True}

            # Write metadata to the json file
            FileHandler.save_json(data=metadata, file_path=metadata_file_path)

            logger.info(f"Project '{project_name}' created at {project_folder_path.resolve()}.")

    def trainer(self, task: Optional[Union[str, Task]] = None, yaml_path: Optional[str] = None) -> Trainer:
        """Initialize and return a Trainer instance.

        Args:
            task (Union[str, Task], optional): Type of task (classification, detection, segmentation).
            yaml_path (str, optional): Path to the YAML configuration file.

        Returns:
            Trainer: Initialized Trainer instance.
        """
        return Trainer(token_handler=self.token_handler, task=task, yaml_path=yaml_path)

    def compressor_v2(self) -> CompressorV2:
        """Initialize and return a Compressor instance.

        Returns:
            Compressor: Initialized Compressor instance.
        """
        return CompressorV2(token_handler=self.token_handler)

    def converter_v2(self) -> ConverterV2:
        """Initialize and return a Converter instance.

        Returns:
            Converter: Initialized Converter instance.
        """
        return ConverterV2(token_handler=self.token_handler, user_info=self.user_info)

    def quantizer(self) -> Quantizer:
        """Initialize and return a Quantizer instance.

        Returns:
            Quantizer: Initialized Quantizer instance.
        """
        return Quantizer(token_handler=self.token_handler, user_info=self.user_info)

    def benchmarker_v2(self) -> BenchmarkerV2:
        """Initialize and return a Benchmarker instance.

        Returns:
            Benchmarker: Initialized Benchmarker instance.
        """
        return BenchmarkerV2(token_handler=self.token_handler, user_info=self.user_info)

    def np_inferencer(self, config_path: str, input_model_path: str) -> NPInferencer:
        """Initialize and return a Inferencer instance.

        Returns:
            Inferencer: Initialized Inferencer instance.
        """

        return NPInferencer(config_path=config_path, input_model_path=input_model_path)

    def custom_inferencer(self, input_model_path: str) -> CustomInferencer:
        """Initialize and return a Inferencer instance.

        Returns:
            Inferencer: Initialized Inferencer instance.
        """
        return CustomInferencer(input_model_path=input_model_path)


class TAO:
    def __init__(self, ngc_api_key: str) -> None:
        """Initialize TAO instance and perform user authentication.

        Args:
            ngc_api_key (str): API key for TAO authentication.
        """
        self.ngc_api_key = ngc_api_key
        self.token_handler = TAOTokenHandler(ngc_api_key=ngc_api_key)

    def trainer(self) -> TAOTrainer:
        """Initialize and return a Trainer instance.

        Returns:
            TAO: Initialized Trainer instance.
        """
        return TAOTrainer(token_handler=self.token_handler)


class NPQAI:
    def __init__(self, api_token: str) -> None:
        # Define the command and arguments
        command = "qai-hub"
        args = ["configure", "--api_token", f"{api_token}"]

        # Execute the command
        result = subprocess.run([command] + args, capture_output=True, text=True)
        logger.info(result)

    def converter(self) -> NPQAIConverter:
        """Initialize and return a Converter instance.

        Returns:
            NPQAIConverter: Initialized Converter instance.
        """
        return NPQAIConverter()

    def benchmarker(self) -> NPQAIBenchmarker:
        """Initialize and return a Benchmarker instance.

        Returns:
            NPQAIBenchmarker: Initialized Benchmarker instance.
        """
        return NPQAIBenchmarker()

    def quantizer(self) -> NPQAIQuantizer:
        """Initialize and return a Quantizer instance.

        Returns:
            NPQAIQuantizer: Initialized Quantizer instance.
        """
        return NPQAIQuantizer()



from pathlib import Path
from typing import List, Optional, Union

import qai_hub as hub
from loguru import logger
from qai_hub import JobStatus, QuantizeDtype
from qai_hub.client import Dataset, QuantizeJob
from qai_hub.public_rest_api import DatasetEntries

from netspresso.analytics import netspresso_analytics
from netspresso.enums import Status
from netspresso.metadata.quantizer import NPQAIQuantizerMetadata
from netspresso.np_qai.base import NPQAIBase
from netspresso.np_qai.options.quantize import QuantizeOptions
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class NPQAIQuantizer(NPQAIBase):
    def get_quantize_task_status(self, quantize_task_id: str) -> JobStatus:
        """
        Get the status of a quantize task.

        Args:
            quantize_task_id: The ID of the quantize task to get the status of.

        Returns:
            JobStatus: The status of the quantize task.

        Note:
            For details, see `JobStatus in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.JobStatus.html>`_.
        """
        job: QuantizeJob = hub.get_job(quantize_task_id)
        status = job.get_status()

        return status

    def update_quantize_task(self, metadata: NPQAIQuantizerMetadata) -> NPQAIQuantizerMetadata:
        """
        Update the quantize task.

        Args:
            metadata: The metadata of the quantize task.

        Returns:
            NPQAIQuantizerMetadata: The updated metadata of the quantize task.
        """
        job: QuantizeJob = hub.get_job(metadata.quantize_info.quantize_task_uuid)
        status = job.wait()

        if status.success:
            logger.info(f"{status.symbol} {status.state.name}")
            self.download_model(job=job, filename=metadata.quantized_model_path)
            target_model = job.get_target_model()
            metadata.quantize_info.output_model_uuid = target_model.model_id
            metadata.status = Status.COMPLETED
        elif status.failure:
            logger.info(f"{status.symbol} {status.state}: {status.message}")
            metadata.status = Status.ERROR
            metadata.update_message(exception_detail=status.message)

        MetadataHandler.save_metadata(data=metadata, folder_path=Path(metadata.quantized_model_path).parent.as_posix())

        return metadata

    def quantize_model(
        self,
        input_model_path: Union[str, Path],
        output_dir: str,
        weights_dtype: QuantizeDtype,
        activations_dtype: QuantizeDtype,
        options: Union[QuantizeOptions, str] = QuantizeOptions(),
        job_name: Optional[str] = None,
        calibration_data: Union[Dataset, DatasetEntries, str, None] = None,
    ) -> Union[NPQAIQuantizerMetadata, List[NPQAIQuantizerMetadata]]:
        """
        Quantize a model in the QAI hub.

        Args:
            input_model_path: The path to the input model.
            output_dir: The directory to save the quantized model.
            weights_dtype: The data type to use for the weights.
            activations_dtype: The data type to use for the activations.
            options: The options to use for the quantization.
            job_name: The name of the job.
            calibration_data: The calibration data to use for the quantization.

        Returns:
            Union[NPQAIQuantizerMetadata, List[NPQAIQuantizerMetadata]]: Returns a quantizer metadata object if successful.

        Note:
            For details, see `submit_quantize_job in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.submit_quantize_job.html>`_.
        """
        netspresso_analytics.send_event(
            event_name="quantize_model_using_qai",
            event_params={
                "weights_dtype": weights_dtype.name if weights_dtype else "",
                "activations_dtype": activations_dtype.name if activations_dtype else "",
            },
        )

        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        default_model_path = (Path(output_dir) / f"{Path(output_dir).name}.ext").resolve()
        metadata = NPQAIQuantizerMetadata()
        metadata.input_model_path = Path(input_model_path).resolve().as_posix()
        extension = self.get_source_extension(model_path=input_model_path)
        metadata.model_info.framework = self.get_framework(extension=extension)

        MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        try:
            quantized_model_path = default_model_path.with_suffix(".onnx").as_posix()

            cli_string = options.to_cli_string() if isinstance(options, QuantizeOptions) else options

            job = hub.submit_quantize_job(
                model=input_model_path,
                calibration_data=calibration_data,
                weights_dtype=weights_dtype,
                activations_dtype=activations_dtype,
                name=job_name,
                options=cli_string,
            )
            metadata.quantized_model_path = quantized_model_path
            metadata.quantize_info.quantize_task_uuid = job.job_id
            metadata.quantize_info.input_model_uuid = job.model.model_id

            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        except KeyboardInterrupt:
            metadata.status = Status.STOPPED
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def download_model(self, job: QuantizeJob, filename: str):
        """
        Download a model from the QAI hub.

        Args:
            job: The job to download the model from.
            filename: The filename to save the model to.

        Note:
            For details, see `download_target_model in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/generated/qai_hub.QuantizeJob.html#qai_hub.QuantizeJob.download_target_model>`_.
        """
        job.download_target_model(filename=filename)



from pathlib import Path

from netspresso.clients.utils.common import read_file_bytes
from netspresso.clients.utils.requester import Requester


class DatasetAPI:
    def __init__(self, url: str):
        self.url = url

    def get_datasets(self, user_id, headers, skip=None, size=None, sort=None, name=None, format=None, type=None):
        endpoint = f"{self.url}/users/{user_id}/datasets"
        params = {
            key: value
            for key, value in {
                "skip": skip,
                "size": size,
                "sort": sort,
                "name": name,
                "format": format,
                "type": type,
            }.items()
            if value is not None
        }

        response = Requester.get(url=endpoint, params=params, headers=headers)

        return response.json()

    def create_dataset(self, user_id, request_body, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets"

        response = Requester.post_as_json(url=endpoint, request_body=request_body, headers=headers)

        return response.json()

    def delete_dataset(self, user_id, dataset_id, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}"

        response = Requester.delete(url=endpoint, headers=headers)

        return response.json()

    def get_dataset(self, user_id, dataset_id, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}"

        response = Requester.get(url=endpoint, headers=headers)

        return response.json()

    def partial_update_dataset(self, user_id, dataset_id, request_body, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}"

        response = Requester.patch(url=endpoint, request_body=request_body, headers=headers)

        return response.json()

    def update_dataset(self, user_id, dataset_id, request_body, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}"

        response = Requester.put(url=endpoint, request_body=request_body, headers=headers)

        return response.json()

    def get_dataset_jobs(self, user_id, dataset_id, headers, skip=None, size=None, sort=None):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/jobs"
        params = {
            key: value
            for key, value in {
                "skip": skip,
                "size": size,
                "sort": sort,
            }.items()
            if value is not None
        }

        response = Requester.get(url=endpoint, params=params, headers=headers)

        return response.json()

    def run_dataset_jobs(self, user_id, dataset_id, request_body, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/jobs"

        response = Requester.post_as_json(url=endpoint, request_body=request_body, headers=headers)

        return response.json()

    def delete_dataset_job(self, user_id, dataset_id, job_id, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/jobs/{job_id}"

        response = Requester.delete(url=endpoint, headers=headers)

        return response.json()

    def get_dataset_job(self, user_id, dataset_id, job_id, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/jobs/{job_id}"

        response = Requester.get(url=endpoint, headers=headers)

        return response.json()

    def cancel_dataset_job(self, user_id, dataset_id, job_id, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/jobs/{job_id}:cancel"

        response = Requester.post_as_json(url=endpoint, headers=headers)

        return response.json()

    def download_job_artifacts(self, user_id, dataset_id, job_id, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/jobs/{job_id}:download"

        response = Requester.get(url=endpoint, headers=headers)

        return response.json()

    def get_specs_schema(self, user_id, dataset_id, action, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}/specs/{action}/schema"

        response = Requester.get(url=endpoint, headers=headers)

        return response.json()

    def upload_dataset(self, user_id, dataset_id, dataset_path, headers):
        endpoint = f"{self.url}/users/{user_id}/datasets/{dataset_id}:upload"
        file_content = read_file_bytes(file_path=dataset_path)
        file_obj = [("file", (Path(dataset_path).name, file_content))]

        response = Requester.post_as_form(url=endpoint, binary=file_obj, headers=headers)

        return response.json()



import tempfile
from dataclasses import is_dataclass
from pathlib import Path
from typing import Union

from omegaconf import OmegaConf

from netspresso.trainer.augmentations import AugmentationConfig
from netspresso.trainer.data import DatasetConfig
from netspresso.trainer.models import ModelConfig
from netspresso.trainer.training import EnvironmentConfig, LoggingConfig, ScheduleConfig


class TrainerConfigs:
    def __init__(
        self,
        data: Union[DatasetConfig, Path],
        augmentation: Union[AugmentationConfig, Path],
        model: Union[ModelConfig, Path],
        training: Union[ScheduleConfig, Path],
        logging: Union[LoggingConfig, Path],
        environment: Union[EnvironmentConfig, Path],
        prefix: str = "temp_np_trainer_configs_",
    ) -> None:
        """Initialize TrainerConfigs with specified configurations.

        Args:
            data (Union[DatasetConfig, Path]): Dataset configuration or path to a YAML file.
            augmentation (Union[AugmentationConfig, Path]): Augmentation configuration or path to a YAML file.
            model (Union[ModelConfig, Path]): Model configuration or path to a YAML file.
            training (Union[ScheduleConfig, Path]): Training schedule configuration or path to a YAML file.
            logging (Union[LoggingConfig, Path]): Logging configuration or path to a YAML file.
            environment (Union[EnvironmentConfig, Path]): Environment configuration or path to a YAML file.
            prefix (str, optional): Prefix for the temporary folder. Defaults to "temp_np_trainer_configs_".
        """

        self.prefix = prefix
        self.temp_folder = self.create_temp_folder()
        self.data = self.handle_config(data, "data")
        self.augmentation = self.handle_config(augmentation, "augmentation")
        self.model = self.handle_config(model, "model")
        self.training = self.handle_config(training, "training")
        self.logging = self.handle_config(logging, "logging")
        self.environment = self.handle_config(environment, "environment")

    def create_temp_folder(self) -> str:
        """Create a temporary folder and return its path.

        Returns:
            str: Path to the created temporary folder.
        """

        return tempfile.mkdtemp(prefix=self.prefix)

    def _save_config_as_yaml(self, config, type: str) -> Path:
        """Save the given configuration as a YAML file in the temporary folder.

        Args:
            config: Configuration to be saved.
            type (str): Type of the configuration.

        Returns:
            Path: Path to the saved YAML file.
        """

        config = OmegaConf.create({type: config})
        yaml_path = Path(self.temp_folder) / f"{type}.yaml"
        OmegaConf.save(config=config, f=yaml_path)

        return yaml_path

    def handle_config(self, config, type: str) -> Path:
        """Handle the input configuration, either saving it as a YAML file or using YAML file path directly.

        Args:
            config: Input configuration.
            type (str): Type of the configuration.

        Returns:
            Path: Path to the saved YAML file if the input is a dataclass; otherwise, the YAML file path itself.
        """

        if is_dataclass(config):
            return self._save_config_as_yaml(config, type)
        else:
            return config



import time
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Optional, Union
from urllib import request

from loguru import logger

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.launcher import launcher_client_v2
from netspresso.clients.launcher.v2.schemas.task.quantize.request_body import (
    AutomaticQuantizeOption,
    CustomQuantizeOption,
    PlainQuantizationOption,
    RecommendationOption,
)
from netspresso.clients.launcher.v2.schemas.task.quantize.response_body import QuantizeTask
from netspresso.enums import (
    QuantizationMode,
    QuantizationPrecision,
    ServiceTask,
    SimilarityMetric,
    Status,
    TaskStatusForDisplay,
)
from netspresso.metadata.quantizer import QuantizerMetadata
from netspresso.quantizer.schema import PrecisionByLayer, PrecisionByOperator, RecommendationPrecisions
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class Quantizer(NetsPressoBase):
    def __init__(self, token_handler: TokenHandler, user_info: UserResponse):
        """Initialize the Quantizer."""

        super().__init__(token_handler)
        self.user_info = user_info

    def initialize_metadata(
        self,
        output_dir,
        input_model_path,
        threshold,
        quantization_mode,
        metric,
        weight_precision,
        activation_precision,
    ):
        def create_metadata_with_status(status, error_message=None):
            metadata = QuantizerMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = QuantizerMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Quantization task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.input_model_path = Path(input_model_path).resolve().as_posix()
            metadata.quantize_info.threshold = threshold
            metadata.quantize_info.quantization_mode = quantization_mode
            metadata.quantize_info.metric = metric
            metadata.quantize_info.weight_precision = weight_precision
            metadata.quantize_info.activation_precision = activation_precision
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def _download_quantized_model(
        self, quantize_task: QuantizeTask, output_dir: str, metadata: QuantizerMetadata
    ) -> None:
        """Download the quantizeed model with given quantization task or quantization task uuid.

        Args:
            quantization_task (QuantizeTask): Launcher Model Object or the uuid of the quantization task.

        Raises:
            e: If an error occurs while getting the quantization task information.
        """

        self.token_handler.validate_token()

        try:
            download_url = launcher_client_v2.quantizer.download_model_file(
                quantize_task_uuid=quantize_task.quantize_task_id,
                access_token=self.token_handler.tokens.access_token,
            ).data.presigned_download_url

            default_model_path = FileHandler.get_default_model_path(folder_path=output_dir)
            download_model_path = default_model_path.with_suffix(".zip").as_posix()

            request.urlretrieve(download_url, download_model_path)
            logger.info(f"Model downloaded at {Path(download_model_path)}")

            metadata.quantized_model_path = download_model_path
            FileHandler.unzip(zip_file_path=download_model_path, target_path=output_dir)
            FileHandler.remove_file(file_path=download_model_path)

            old_file_path = Path(output_dir) / "quantized_qdq.onnx"
            quantized_model_path = default_model_path.with_suffix(".onnx").as_posix()
            metadata.quantized_model_path = old_file_path
            FileHandler.rename_file(old_file_path=old_file_path, new_file_path=quantized_model_path)

            compare_result = FileHandler.load_json(file_path=output_dir / "snr_compare_result.json")

            self.print_remaining_credit(service_task=ServiceTask.MODEL_QUANTIZE)

            metadata.status = Status.COMPLETED
            metadata.quantized_model_path = quantized_model_path
            metadata.compare_result = compare_result

            return metadata

        except Exception as e:
            logger.error(f"Download quantized model failed. Error: {e}")
            raise e

    def _download_recommendation_result(
        self, quantize_task: QuantizeTask, output_dir: str, metadata: QuantizerMetadata
    ) -> None:
        self.token_handler.validate_token()

        try:
            download_url = launcher_client_v2.quantizer.download_model_file(
                quantize_task_uuid=quantize_task.quantize_task_id,
                access_token=self.token_handler.tokens.access_token,
            ).data.presigned_download_url

            download_path = (Path(output_dir) / "custom_quantization_suggestion.json").resolve().as_posix()

            request.urlretrieve(download_url, download_path)
            logger.info(f"Model downloaded at {Path(download_path)}")

            self.print_remaining_credit(service_task=ServiceTask.MODEL_QUANTIZE)

            metadata.status = Status.COMPLETED
            metadata.recommendation_result_path = download_path

            return metadata

        except Exception as e:
            logger.error(f"Download quantized model failed. Error: {e}")
            raise e

    def _upload_model(self, input_model_path: str):
        # Get presigned_model_upload_url
        presigned_url_response = launcher_client_v2.quantizer.presigned_model_upload_url(
            access_token=self.token_handler.tokens.access_token,
            input_model_path=input_model_path,
        )

        # Upload model_file
        launcher_client_v2.quantizer.upload_model_file(
            access_token=self.token_handler.tokens.access_token,
            input_model_path=input_model_path,
            presigned_upload_url=presigned_url_response.data.presigned_upload_url,
        )

        # Validate model_file
        validate_model_response = launcher_client_v2.quantizer.validate_model_file(
            access_token=self.token_handler.tokens.access_token,
            input_model_path=input_model_path,
            ai_model_id=presigned_url_response.data.ai_model_id,
        )

        return validate_model_response

    def _quantize_model(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        quantization_mode: QuantizationMode,
        quantization_options: Union[PlainQuantizationOption, CustomQuantizeOption, AutomaticQuantizeOption],
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> QuantizerMetadata:
        """Quantize a model to the specified framework.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the quantized model.
            dataset_path (str): Path to the dataset. Useful for certain quantizations.
            quantization_mode (QuantizationMode): Quantization mode
            input_layers (List[InputShape], optional): Target input shape for quantization (e.g., dynamic batch to static batch).
            wait_until_done (bool): If True, wait for the quantization result before returning the function.
                                If False, request the quantization and return  the function immediately.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizerMetadata: Quantize metadata.
        """

        FileHandler.check_input_model_path(input_model_path)
        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata = self.initialize_metadata(
            output_dir=output_dir,
            input_model_path=input_model_path,
            threshold=0,
            quantization_mode=quantization_mode,
            metric=quantization_options.metric,
            weight_precision=QuantizationPrecision.INT8,
            activation_precision=QuantizationPrecision.INT8,
        )

        try:
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            self.validate_token_and_check_credit(service_task=ServiceTask.MODEL_QUANTIZE)

            uploaded_model_response = self._upload_model(input_model_path=input_model_path)

            # Start quantize task
            input_layers = input_layers if input_layers else uploaded_model_response.data.detail.input_layers
            quantize_response = launcher_client_v2.quantizer.start_task(
                access_token=self.token_handler.tokens.access_token,
                input_model_id=uploaded_model_response.data.ai_model_id,
                quantization_mode=quantization_mode,
                quantization_options=quantization_options,
                input_layers=input_layers,
                dataset_path=dataset_path,
            )

            metadata.model_info = uploaded_model_response.data.to()
            metadata.quantize_info = quantize_response.data.to(uploaded_model_response.data.uploaded_file_name)
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

            if wait_until_done:
                while True:
                    self.token_handler.validate_token()
                    quantize_response = launcher_client_v2.quantizer.read_task(
                        access_token=self.token_handler.tokens.access_token,
                        task_id=quantize_response.data.quantize_task_id,
                    )
                    if quantize_response.data.status in [
                        TaskStatusForDisplay.FINISHED,
                        TaskStatusForDisplay.ERROR,
                        TaskStatusForDisplay.TIMEOUT,
                    ]:
                        break

                    time.sleep(sleep_interval)

            if quantize_response.data.status == TaskStatusForDisplay.FINISHED:
                if quantize_response.data.quantization_mode in [
                    "plain_quantization",
                    "custom_quantization",
                    "automatic_quantization",
                ]:
                    metadata = self._download_quantized_model(quantize_response.data, output_dir, metadata)
                elif quantize_response.data.quantization_mode in [QuantizationMode.RECOMMEND_QUANTIZATION]:
                    metadata = self._download_recommendation_result(quantize_response.data, output_dir, metadata)

            else:
                metadata = self.handle_error(metadata, ServiceTask.MODEL_QUANTIZE, quantize_response.data.error_log)

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.MODEL_QUANTIZE, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.MODEL_QUANTIZE)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def uniform_precision_quantization(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        metric: SimilarityMetric = SimilarityMetric.SNR,
        weight_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        activation_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ):
        """Apply uniform precision quantization to a model, specifying precision for weight & activation.

        This method quantizes all layers in the model uniformly based on the specified precision levels for weights and activations.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the quantized model.
            dataset_path (str): Path to the dataset. Useful for certain quantizations.
            metric (SimilarityMetric): Quantization quality metrics.
            weight_precision (QuantizationPrecision): Weight precision
            activation_precision (QuantizationPrecision): Activation precision
            input_layers (List[InputShape], optional): Target input shape for quantization (e.g., dynamic batch to static batch).
            wait_until_done (bool): If True, wait for the quantization result before returning the function.
                                If False, request the quantization and return  the function immediately.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizerMetadata: Quantize metadata.
        """
        netspresso_analytics.send_event(
            event_name="uniform_precision_quantization_using_np",
            event_params={
                "weight_precision": weight_precision,
                "activation_precision": activation_precision,
                "metric": metric,
            },
        )

        quantization_options = PlainQuantizationOption(
            metric=metric,
            weight_precision=weight_precision,
            activation_precision=activation_precision,
        )

        metadata = self._quantize_model(
            input_model_path=input_model_path,
            output_dir=output_dir,
            dataset_path=dataset_path,
            quantization_mode=QuantizationMode.UNIFORM_PRECISION_QUANTIZATION,
            quantization_options=quantization_options,
            input_layers=input_layers,
            wait_until_done=wait_until_done,
            sleep_interval=sleep_interval,
        )

        logger.info("Plain quantization task was completed successfully.")

        return metadata

    def automatic_quantization(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        weight_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        activation_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        metric: SimilarityMetric = SimilarityMetric.SNR,
        threshold: Union[float, int] = 0,
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> QuantizerMetadata:
        """Apply automatic quantization to a model, specifying precision for weight & activation.

        This method quantizes layers in the model based on the specified precision levels for weights and activations, while evaluating
        the quality of quantization using the defined metric. Only layers that meet the specified quality `threshold` are quantized;
        layers that do not meet this threshold remain unquantized to preserve model accuracy.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the quantized model.
            dataset_path (str): Path to the dataset. Useful for certain quantizations.
            weight_precision (QuantizationPrecision): Weight precision
            activation_precision (QuantizationPrecision): Activation precision
            metric (SimilarityMetric): Quantization quality metrics.
            threshold (Union[float, int]): Quality threshold for quantization. Layers that do not meet this threshold based on the metric are not quantized.
            input_layers (List[InputShape], optional): Target input shape for quantization (e.g., dynamic batch to static batch).
            wait_until_done (bool): If True, wait for the quantization result before returning the function.
                                If False, request the quantization and return  the function immediately.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizerMetadata: Quantize metadata.
        """
        netspresso_analytics.send_event(
            event_name="automatic_quantization_using_np",
            event_params={
                "weight_precision": weight_precision,
                "activation_precision": activation_precision,
                "metric": metric,
                "threshold": threshold,
            },
        )

        quantization_options = AutomaticQuantizeOption(
            metric=metric,
            threshold=threshold,
            weight_precision=weight_precision,
            activation_precision=activation_precision,
        )

        metadata = self._quantize_model(
            input_model_path=input_model_path,
            output_dir=output_dir,
            dataset_path=dataset_path,
            quantization_mode=QuantizationMode.AUTOMATIC_QUANTIZATION,
            quantization_options=quantization_options,
            input_layers=input_layers,
            wait_until_done=wait_until_done,
            sleep_interval=sleep_interval,
        )

        logger.info("Automatic quantization task was completed successfully.")

        return metadata

    def _custom_quantization(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        custom_quantization_dictionary: Dict,
        metric: SimilarityMetric = SimilarityMetric.SNR,
        weight_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        activation_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> QuantizerMetadata:
        quantization_options = CustomQuantizeOption(
            metric=metric,
            custom_precision=custom_quantization_dictionary,
            weight_precision=weight_precision,
            activation_precision=activation_precision,
        )

        metadata = self._quantize_model(
            input_model_path=input_model_path,
            output_dir=output_dir,
            dataset_path=dataset_path,
            quantization_mode=QuantizationMode.CUSTOM_PRECISION_QUANTIZATION,
            quantization_options=quantization_options,
            input_layers=input_layers,
            wait_until_done=wait_until_done,
            sleep_interval=sleep_interval,
        )

        return metadata

    def custom_precision_quantization_by_layer_name(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        precision_by_layer_name: List[PrecisionByLayer],
        default_weight_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        default_activation_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        metric: SimilarityMetric = SimilarityMetric.SNR,
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> QuantizerMetadata:
        """
        Apply custom precision quantization to a model, specifying precision for each layer name.

        This function allows precise control over the quantization process by enabling the user to
        specify quantization precision (e.g., INT8, FP16) for each named layer within the model.
        The `precision_by_layer_name` parameter provides a list where each item details the target
        precision for a specific layer name, enabling customized quantization that can enhance
        model performance or compatibility.

        Users can target specific layers to be quantized to lower precision for optimized model
        size and performance while keeping critical layers at higher precision for accuracy.
        Layers not explicitly listed in `precision_by_layer_name` will use
        `default_weight_precision` and `default_activation_precision`.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the quantized model.
            dataset_path (str): Path to the dataset. Useful for certain quantizations.
            precision_by_layer_name (List[PrecisionByLayer]):
                List of `PrecisionByLayer` objects that specify the desired precision for each
                layer name in the model. Each entry includes: `name` (str): The layer name (e.g., /backbone/conv_first/block/act/Mul_output_0).
                `precision` (QuantizationPrecision): The quantization precision level.
            default_weight_precision (QuantizationPrecision): Weight precision.
            default_activation_precision (QuantizationPrecision): Activation precision.
            metric (SimilarityMetric): Quantization quality metrics.
            input_layers (List[InputShape], optional):
                Target input shape for quantization (e.g., dynamic batch to static batch).
            wait_until_done (bool): If True, wait for the quantization result before returning
                the function. If False, request the quantization and return immediately.
            sleep_interval (int): Interval in seconds between checks when `wait_until_done` is True.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizerMetadata: Quantization metadata containing status, paths, etc.
        """
        netspresso_analytics.send_event(
            event_name="custom_precision_quantization_by_layer_name_using_np",
            event_params={
                "weight_precision": default_weight_precision,
                "activation_precision": default_activation_precision,
                "metric": metric,
            },
        )

        layers = {layer.name: layer.precision for layer in precision_by_layer_name}

        custom_quantization_dictionary = {"layers": layers, "operators": {}}

        metadata = self._custom_quantization(
            input_model_path=input_model_path,
            output_dir=output_dir,
            dataset_path=dataset_path,
            custom_quantization_dictionary=custom_quantization_dictionary,
            metric=metric,
            weight_precision=default_weight_precision,
            activation_precision=default_activation_precision,
            input_layers=input_layers,
            wait_until_done=wait_until_done,
            sleep_interval=sleep_interval,
        )

        logger.info("Custom quantization by layer name task was completed successfully.")

        return metadata

    def custom_precision_quantization_by_operator_type(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        precision_by_operator_type: List[PrecisionByOperator],
        default_weight_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        default_activation_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        metric: SimilarityMetric = SimilarityMetric.SNR,
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> QuantizerMetadata:
        """
        Apply custom quantization to a model, specifying precision for each operator type.

        This function allows for highly customizable quantization by enabling the user to specify
        the quantization precision (e.g., INT8, FP16) for each operator type within a model. The
        `precision_by_operator_type` parameter is a list of mappings where each entry indicates
        the quantization precision for a specific operator type, such as convolution (Conv),
        matrix multiplication (MatMul), etc.

        Using `precision_by_operator_type`, users can selectively fine-tune the quantization
        strategy for different operators within the model, based on performance requirements
        or hardware capabilities. Operators not explicitly specified in
        `precision_by_operator_type` will fall back to `default_weight_precision` and
        `default_activation_precision`.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the quantized model.
            dataset_path (str): Path to the dataset. Useful for certain quantizations.
            precision_by_operator_type (List[PrecisionByOperator]):
                List of `PrecisionByOperator` objects that specify the desired precision for each
                operator type in the model. Each entry includes: `type` (str): The operator type (e.g., Conv, MatMul). `precision` (QuantizationPrecision): The quantization precision level.
            default_weight_precision (QuantizationPrecision): Weight precision.
            default_activation_precision (QuantizationPrecision): Activation precision.
            metric (SimilarityMetric): Quantization quality metrics.
            input_layers (List[InputShape], optional):
                Target input shape for quantization (e.g., dynamic batch to static batch).
            wait_until_done (bool): If True, wait for the quantization result before returning
                the function. If False, request the quantization and return immediately.
            sleep_interval (int): Interval in seconds between checks when `wait_until_done` is True.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizerMetadata: Quantization metadata containing status, paths, etc.
        """
        netspresso_analytics.send_event(
            event_name="custom_precision_quantization_by_operator_type_using_np",
            event_params={
                "weight_precision": default_weight_precision,
                "activation_precision": default_activation_precision,
                "metric": metric,
            },
        )

        operators = {layer.type: layer.precision for layer in precision_by_operator_type}
        custom_quantization_dictionary = {"layers": {}, "operators": operators}

        metadata = self._custom_quantization(
            input_model_path=input_model_path,
            output_dir=output_dir,
            dataset_path=dataset_path,
            custom_quantization_dictionary=custom_quantization_dictionary,
            metric=metric,
            weight_precision=default_weight_precision,
            activation_precision=default_activation_precision,
            input_layers=input_layers,
            wait_until_done=wait_until_done,
            sleep_interval=sleep_interval,
        )

        logger.info("Custom quantization by operator type task was completed successfully.")

        return metadata

    def get_recommendation_precision(
        self,
        input_model_path: str,
        output_dir: str,
        dataset_path: Optional[str],
        weight_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        activation_precision: QuantizationPrecision = QuantizationPrecision.INT8,
        metric: SimilarityMetric = SimilarityMetric.SNR,
        threshold: Union[float, int] = 0,
        input_layers: List[Dict[str, int]] = None,
        wait_until_done: bool = True,
        sleep_interval: int = 30,
    ) -> QuantizerMetadata:
        """Get recommended precision for a model based on a specified quality threshold.

        This function analyzes each layer of the given model and recommends precision settings
        for layers that do not meet the specified threshold, helping to balance quantization
        quality and performance.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local folder path to save the quantized model.
            dataset_path (str): Path to the dataset. Useful for certain quantizations.
            weight_precision (QuantizationPrecision): Target precision for weights.
            activation_precision (QuantizationPrecision): Target precision for activations.
            metric (SimilarityMetric): Metric used to evaluate quantization quality.
            threshold (Union[float, int]): Quality threshold; layers below this threshold will
                            receive precision recommendations.
            input_layers (List[Dict[str, int]], optional): Specifications for input shapes
                            (e.g., to convert from dynamic to static batch size).
            wait_until_done (bool): If True, waits for the quantization process to finish
                            before returning. If False, starts the process and returns immediately.
            sleep_interval (int): Interval, in seconds, between checks when `wait_until_done`
                            is True.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizerMetadata: Quantize metadata.

        """
        netspresso_analytics.send_event(
            event_name="get_recommendation_precision_using_np",
            event_params={
                "weight_precision": weight_precision,
                "activation_precision": activation_precision,
                "metric": metric,
                "threshold": threshold,
            },
        )

        quantization_options = RecommendationOption(
            metric=metric,
            threshold=threshold,
            weight_precision=weight_precision,
            activation_precision=activation_precision,
        )

        metadata = self._quantize_model(
            input_model_path=input_model_path,
            output_dir=output_dir,
            dataset_path=dataset_path,
            quantization_mode=QuantizationMode.RECOMMEND_QUANTIZATION,
            quantization_options=quantization_options,
            input_layers=input_layers,
            wait_until_done=wait_until_done,
            sleep_interval=sleep_interval,
        )

        logger.info("Get recommendation precision task was completed successfully.")

        return metadata

    def load_recommendation_precision_result(self, file_path: str):
        recommendation_result = FileHandler.load_json(file_path=file_path)
        layers = recommendation_result["layers"]
        operators = recommendation_result["operators"]

        precision_by_layer_name = [
            PrecisionByLayer(
                name=layer["name"],
                precision=layer["recommendation"]["precision"][0],
            )
            for layer in layers
        ]
        precision_by_operator_type = [
            PrecisionByOperator(
                type=operator["type"],
                precision=operator["recommendation"]["precision"][0],
            )
            for operator in operators
        ]

        recommendation_precisions = RecommendationPrecisions(
            layers=precision_by_layer_name,
            operators=precision_by_operator_type,
        )

        return recommendation_precisions

    def get_quantization_task(self, quantization_task_id: str) -> QuantizeTask:
        """Get the quantization task information with given quantization task uuid.

        Args:
            quantization_task_id (str): Quantize task UUID of the quantize task.

        Raises:
            e: If an error occurs during the model quantization.

        Returns:
            QuantizeTask: Model quantization task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.quantizer.read_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=quantization_task_id,
        )
        return response.data

    def cancel_quantization_task(self, quantization_task_id: str) -> QuantizeTask:
        """Cancel the quantization task with given quantization task uuid.

        Args:
            quantization_task_id (str): Quantize task UUID of the quantize task.

        Raises:
            e: If an error occurs during the task cancel.

        Returns:
            QuantizeTask: Model quantization task dictionary.
        """

        self.token_handler.validate_token()

        response = launcher_client_v2.quantizer.cancel_task(
            access_token=self.token_handler.tokens.access_token,
            task_id=quantization_task_id,
        )
        return response.data



from dataclasses import dataclass, field
from typing import List

from netspresso.enums import DataType, DeviceName, Framework, SoftwareVersion, TaskType
from netspresso.metadata.common import AvailableOption, BaseMetadata, ModelInfo


@dataclass
class ConvertInfo:
    convert_task_uuid: str = ""
    framework: Framework = ""
    display_framework: str = ""
    device_name: DeviceName = ""
    display_device_name: str = ""
    display_brand_name: str = ""
    data_type: DataType = ""
    software_version: SoftwareVersion = ""
    display_software_version: str = ""
    model_file_name: str = ""
    input_model_uuid: str = ""
    output_model_uuid: str = ""


@dataclass
class ConverterMetadata(BaseMetadata):
    task_type: TaskType = TaskType.CONVERT
    input_model_path: str = ""
    converted_model_path: str = ""
    model_info: ModelInfo = field(default_factory=ModelInfo)
    convert_task_info: ConvertInfo = field(default_factory=ConvertInfo)
    available_options: List[AvailableOption] = field(default_factory=list)



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class EfficientFormerArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "efficientformer",
            "params": {
                "num_attention_heads": 8,
                "attention_channels": 256,
                "attention_dropout_prob": 0.0,
                "attention_value_expansion_ratio": 4,
                "ffn_intermediate_ratio": 4,
                "ffn_dropout_prob": 0.0,
                "ffn_act_type": "gelu",
                "vit_num": 1,
            },
            "stage_params": [
                {"num_blocks": 3, "channels": 48},
                {"num_blocks": 2, "channels": 96},
                {"num_blocks": 6, "channels": 224},
                {"num_blocks": 4, "channels": 448},
            ],
        }
    )


@dataclass
class ClassificationEfficientFormerModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "efficientformer_l1"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: EfficientFormerArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class SegmentationEfficientFormerModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "efficientformer_l1"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: EfficientFormerArchitectureConfig(
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )


@dataclass
class DetectionEfficientFormerModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "efficientformer_l1"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: EfficientFormerArchitectureConfig(
            neck={
                "name": "fpn",
                "params": {
                    "num_outs": 4,
                    "start_level": 0,
                    "end_level": -1,
                    "add_extra_convs": False,
                    "relu_before_extra_convs": False,
                },
            },
            head={
                "name": "anchor_decoupled_head",
                "params": {
                    # Anchor parameters
                    "anchor_sizes": [
                        [
                            32,
                        ],
                        [
                            64,
                        ],
                        [
                            128,
                        ],
                        [
                            256,
                        ],
                    ],
                    "aspect_ratios": [0.5, 1.0, 2.0],
                    "num_layers": 1,
                    "norm_type": "batch_norm",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "retinanet_loss", "weight": None},
        ]
    )



from netspresso.enums.base import StrEnum


class ExperimentAction(StrEnum):
    TRAIN = "train"
    EVALUATE = "evaluate"
    EXPORT = "export"
    PRUNE = "prune"
    RETRAIN = "retrain"
    GEN_TRT_ENGINE = "gen_trt_engine"
    INFERENCE = "inference"


class ConvertAction(StrEnum):
    DATASET_CONVERT = "dataset_convert"
    CONVERT = "convert"
    CONVERT_AND_INDEX = "convert_and_index"
    CONVERT_EFFICIENTDET_TF1 = "convert_efficientdet_tf1"
    CONVERT_EFFICIENTDET_TF2 = "convert_efficientdet_tf2"
    CONVERT_AUGMENT = "augment"
    CONVERT_KMEANS = "kmeans"
